# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: qm/grpc/v2/job_api.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'qm/grpc/v2/job_api.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from qm.grpc.qm.grpc.v2 import qm_api_pb2 as qm_dot_grpc_dot_v2_dot_qm__api__pb2
from qm.grpc.qm.grpc.v2 import common_types_pb2 as qm_dot_grpc_dot_v2_dot_common__types__pb2
from qm.grpc.qm.pb import inc_qm_api_pb2 as qm_dot_pb_dot_inc__qm__api__pb2
from qm.grpc.qm.pb import job_manager_pb2 as qm_dot_pb_dot_job__manager__pb2
from qm.grpc.qm.pb import job_results_pb2 as qm_dot_pb_dot_job__results__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18qm/grpc/v2/job_api.proto\x12\nqm.grpc.v2\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x17qm/grpc/v2/qm_api.proto\x1a\x1dqm/grpc/v2/common_types.proto\x1a\x16qm/pb/inc_qm_api.proto\x1a\x17qm/pb/job_manager.proto\x1a\x17qm/pb/job_results.proto\"\xc4\x02\n\x1bSetMatrixCorrectionResponse\x12]\n\x07success\x18\x01 \x01(\x0b\x32J.qm.grpc.v2.SetMatrixCorrectionResponse.SetMatrixCorrectionResponseSuccessH\x00\x12Y\n\x05\x65rror\x18\x02 \x01(\x0b\x32H.qm.grpc.v2.SetMatrixCorrectionResponse.SetMatrixCorrectionResponseErrorH\x00\x1a\x33\n SetMatrixCorrectionResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a$\n\"SetMatrixCorrectionResponseSuccessB\x10\n\x0eresponse_oneof\"\xe7\x02\n SetIntermediateFrequencyResponse\x12g\n\x07success\x18\x01 \x01(\x0b\x32T.qm.grpc.v2.SetIntermediateFrequencyResponse.SetIntermediateFrequencyResponseSuccessH\x00\x12\x63\n\x05\x65rror\x18\x02 \x01(\x0b\x32R.qm.grpc.v2.SetIntermediateFrequencyResponse.SetIntermediateFrequencyResponseErrorH\x00\x1a\x38\n%SetIntermediateFrequencyResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a)\n\'SetIntermediateFrequencyResponseSuccessB\x10\n\x0eresponse_oneof\"\xa8\x02\n\x17SetDigitalDelayResponse\x12U\n\x07success\x18\x01 \x01(\x0b\x32\x42.qm.grpc.v2.SetDigitalDelayResponse.SetDigitalDelayResponseSuccessH\x00\x12Q\n\x05\x65rror\x18\x02 \x01(\x0b\x32@.qm.grpc.v2.SetDigitalDelayResponse.SetDigitalDelayResponseErrorH\x00\x1a/\n\x1cSetDigitalDelayResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a \n\x1eSetDigitalDelayResponseSuccessB\x10\n\x0eresponse_oneof\"\xaf\x02\n\x18SetDigitalBufferResponse\x12W\n\x07success\x18\x01 \x01(\x0b\x32\x44.qm.grpc.v2.SetDigitalBufferResponse.SetDigitalBufferResponseSuccessH\x00\x12S\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x42.qm.grpc.v2.SetDigitalBufferResponse.SetDigitalBufferResponseErrorH\x00\x1a\x30\n\x1dSetDigitalBufferResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a!\n\x1fSetDigitalBufferResponseSuccessB\x10\n\x0eresponse_oneof\"\xaf\x02\n\x18SetInputDcOffsetResponse\x12W\n\x07success\x18\x01 \x01(\x0b\x32\x44.qm.grpc.v2.SetInputDcOffsetResponse.SetInputDcOffsetResponseSuccessH\x00\x12S\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x42.qm.grpc.v2.SetInputDcOffsetResponse.SetInputDcOffsetResponseErrorH\x00\x1a\x30\n\x1dSetInputDcOffsetResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a!\n\x1fSetInputDcOffsetResponseSuccessB\x10\n\x0eresponse_oneof\"\xb6\x02\n\x19SetOutputDcOffsetResponse\x12Y\n\x07success\x18\x01 \x01(\x0b\x32\x46.qm.grpc.v2.SetOutputDcOffsetResponse.SetOutputDcOffsetResponseSuccessH\x00\x12U\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x44.qm.grpc.v2.SetOutputDcOffsetResponse.SetOutputDcOffsetResponseErrorH\x00\x1a\x31\n\x1eSetOutputDcOffsetResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\"\n SetOutputDcOffsetResponseSuccessB\x10\n\x0eresponse_oneof\"\x8c\x02\n\x13SetIoValuesResponse\x12M\n\x07success\x18\x01 \x01(\x0b\x32:.qm.grpc.v2.SetIoValuesResponse.SetIoValuesResponseSuccessH\x00\x12I\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x38.qm.grpc.v2.SetIoValuesResponse.SetIoValuesResponseErrorH\x00\x1a+\n\x18SetIoValuesResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\x1c\n\x1aSetIoValuesResponseSuccessB\x10\n\x0eresponse_oneof\"\x85\x02\n\x12SetIoValuesRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12:\n\x03io1\x18\x02 \x01(\x0b\x32-.qm.grpc.v2.SetIoValuesRequest.IOValueSetData\x12:\n\x03io2\x18\x03 \x01(\x0b\x32-.qm.grpc.v2.SetIoValuesRequest.IOValueSetData\x1ag\n\x0eIOValueSetData\x12\x13\n\tint_value\x18\x02 \x01(\x05H\x00\x12\x16\n\x0c\x64ouble_value\x18\x03 \x01(\x01H\x00\x12\x17\n\rboolean_value\x18\x04 \x01(\x08H\x00\x42\x0f\n\roneof_IO_type\"$\n\x12GetIoValuesRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"N\n\x0cIOValuesData\x12\x11\n\tint_value\x18\x01 \x01(\r\x12\x14\n\x0c\x64ouble_value\x18\x02 \x01(\x01\x12\x15\n\rboolean_value\x18\x03 \x01(\x08\"\x89\x04\n\x13GetIoValuesResponse\x12M\n\x07success\x18\x01 \x01(\x0b\x32:.qm.grpc.v2.GetIoValuesResponse.GetIoValuesResponseSuccessH\x00\x12I\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x38.qm.grpc.v2.GetIoValuesResponse.GetIoValuesResponseErrorH\x00\x1a\x98\x02\n\x1aGetIoValuesResponseSuccess\x12T\n\x03io1\x18\x01 \x01(\x0b\x32G.qm.grpc.v2.GetIoValuesResponse.GetIoValuesResponseSuccess.IOValuesData\x12T\n\x03io2\x18\x02 \x01(\x0b\x32G.qm.grpc.v2.GetIoValuesResponse.GetIoValuesResponseSuccess.IOValuesData\x1aN\n\x0cIOValuesData\x12\x11\n\tint_value\x18\x01 \x01(\r\x12\x14\n\x0c\x64ouble_value\x18\x02 \x01(\x01\x12\x15\n\rboolean_value\x18\x03 \x01(\x08\x1a+\n\x18GetIoValuesResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"+\n\x19JobServiceIsPausedRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"\x8a\x02\n\x10IsPausedResponse\x12\x43\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x32.qm.grpc.v2.IsPausedResponse.IsPausedResponseErrorH\x00\x12G\n\x07success\x18\x02 \x01(\x0b\x32\x34.qm.grpc.v2.IsPausedResponse.IsPausedResponseSuccessH\x00\x1a(\n\x15IsPausedResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a,\n\x17IsPausedResponseSuccess\x12\x11\n\tis_paused\x18\x01 \x01(\x08\x42\x10\n\x0eresponse_oneof\",\n\x1aJobServiceGetConfigRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"/\n\x1dJobServiceGetJobStatusRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"\x1f\n\rResumeRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"\xe9\x01\n\x0eResumeResponse\x12\x43\n\x07success\x18\x01 \x01(\x0b\x32\x30.qm.grpc.v2.ResumeResponse.ResumeResponseSuccessH\x00\x12?\n\x05\x65rror\x18\x02 \x01(\x0b\x32..qm.grpc.v2.ResumeResponse.ResumeResponseErrorH\x00\x1a&\n\x13ResumeResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\x17\n\x15ResumeResponseSuccessB\x10\n\x0eresponse_oneof\"d\n\x1aSetMatrixCorrectionRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12*\n\ncorrection\x18\x03 \x01(\x0b\x32\x16.qm.grpc.qm_api.Matrix\"P\n\x1fSetIntermediateFrequencyRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12\x11\n\tfrequency\x18\x03 \x01(\x01\"Q\n\x16SetDigitalDelayRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\t\x12\r\n\x05\x64\x65lay\x18\x04 \x01(\r\"S\n\x17SetDigitalBufferRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\t\x12\x0e\n\x06\x62uffer\x18\x04 \x01(\r\"S\n\x17SetInputDcOffsetRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\t\x12\x0e\n\x06offset\x18\x04 \x01(\x01\"\xc0\x01\n\x18SetOutputDcOffsetRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12\x33\n\nmix_inputs\x18\x03 \x01(\x0b\x32\x1d.qm.grpc.v2.MixInputsDcOffsetH\x00\x12\x37\n\x0csingle_input\x18\x04 \x01(\x0b\x32\x1f.qm.grpc.v2.SingleInputDcOffsetH\x00\x42\x1a\n\x18oneof_element_input_type\"8\n\x1aGetMatrixCorrectionRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\"+\n\x18GetMatrixCorrectionError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\"\x94\x03\n\x1bGetMatrixCorrectionResponse\x12Y\n\x05\x65rror\x18\x01 \x01(\x0b\x32H.qm.grpc.v2.GetMatrixCorrectionResponse.GetMatrixCorrectionResponseErrorH\x00\x12]\n\x07success\x18\x02 \x01(\x0b\x32J.qm.grpc.v2.GetMatrixCorrectionResponse.GetMatrixCorrectionResponseSuccessH\x00\x1aP\n\"GetMatrixCorrectionResponseSuccess\x12*\n\ncorrection\x18\x01 \x01(\x0b\x32\x16.qm.grpc.qm_api.Matrix\x1aW\n GetMatrixCorrectionResponseError\x12\x33\n\x05\x65rror\x18\x01 \x01(\x0b\x32$.qm.grpc.v2.GetMatrixCorrectionErrorB\x10\n\x0eresponse_oneof\"=\n\x1fGetIntermediateFrequencyRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\"\xfa\x02\n GetIntermediateFrequencyResponse\x12\x63\n\x05\x65rror\x18\x01 \x01(\x0b\x32R.qm.grpc.v2.GetIntermediateFrequencyResponse.GetIntermediateFrequencyResponseErrorH\x00\x12g\n\x07success\x18\x02 \x01(\x0b\x32T.qm.grpc.v2.GetIntermediateFrequencyResponse.GetIntermediateFrequencyResponseSuccessH\x00\x1a\x38\n%GetIntermediateFrequencyResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a<\n\'GetIntermediateFrequencyResponseSuccess\x12\x11\n\tfrequency\x18\x01 \x01(\x01\x42\x10\n\x0eresponse_oneof\"B\n\x16GetDigitalDelayRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\t\"\xb7\x02\n\x17GetDigitalDelayResponse\x12U\n\x07success\x18\x01 \x01(\x0b\x32\x42.qm.grpc.v2.GetDigitalDelayResponse.GetDigitalDelayResponseSuccessH\x00\x12Q\n\x05\x65rror\x18\x02 \x01(\x0b\x32@.qm.grpc.v2.GetDigitalDelayResponse.GetDigitalDelayResponseErrorH\x00\x1a/\n\x1cGetDigitalDelayResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a/\n\x1eGetDigitalDelayResponseSuccess\x12\r\n\x05\x64\x65lay\x18\x01 \x01(\rB\x10\n\x0eresponse_oneof\"C\n\x17GetDigitalBufferRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\t\"\xbf\x02\n\x18GetDigitalBufferResponse\x12W\n\x07success\x18\x01 \x01(\x0b\x32\x44.qm.grpc.v2.GetDigitalBufferResponse.GetDigitalBufferResponseSuccessH\x00\x12S\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x42.qm.grpc.v2.GetDigitalBufferResponse.GetDigitalBufferResponseErrorH\x00\x1a\x30\n\x1dGetDigitalBufferResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\x31\n\x1fGetDigitalBufferResponseSuccess\x12\x0e\n\x06\x62uffer\x18\x01 \x01(\rB\x10\n\x0eresponse_oneof\"C\n\x17GetInputDcOffsetRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\t\"(\n\x15GetInputDcOffsetError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\")\n\x17GetInputDcOffsetSuccess\x12\x0e\n\x06offset\x18\x01 \x01(\x01\"\xbf\x02\n\x18GetInputDcOffsetResponse\x12W\n\x07success\x18\x01 \x01(\x0b\x32\x44.qm.grpc.v2.GetInputDcOffsetResponse.GetInputDcOffsetResponseSuccessH\x00\x12S\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x42.qm.grpc.v2.GetInputDcOffsetResponse.GetInputDcOffsetResponseErrorH\x00\x1a\x30\n\x1dGetInputDcOffsetResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\x31\n\x1fGetInputDcOffsetResponseSuccess\x12\x0e\n\x06offset\x18\x01 \x01(\x01\x42\x10\n\x0eresponse_oneof\"6\n\x18GetOutputDcOffsetRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\"\xc1\x03\n\x19GetOutputDcOffsetResponse\x12Y\n\x07success\x18\x01 \x01(\x0b\x32\x46.qm.grpc.v2.GetOutputDcOffsetResponse.GetOutputDcOffsetResponseSuccessH\x00\x12U\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x44.qm.grpc.v2.GetOutputDcOffsetResponse.GetOutputDcOffsetResponseErrorH\x00\x1a\x31\n\x1eGetOutputDcOffsetResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\xac\x01\n GetOutputDcOffsetResponseSuccess\x12\x33\n\nmix_inputs\x18\x02 \x01(\x0b\x32\x1d.qm.grpc.v2.MixInputsDcOffsetH\x00\x12\x37\n\x0csingle_input\x18\x03 \x01(\x0b\x32\x1f.qm.grpc.v2.SingleInputDcOffsetH\x00\x42\x1a\n\x18oneof_element_input_typeB\x10\n\x0eresponse_oneof\"?\n\x11MixInputsDcOffset\x12\x0e\n\x01I\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x0e\n\x01Q\x18\x02 \x01(\x01H\x01\x88\x01\x01\x42\x04\n\x02_IB\x04\n\x02_Q\"%\n\x13SingleInputDcOffset\x12\x0e\n\x06offset\x18\x01 \x01(\x01\"\x8d\x01\n\x1bJobServiceGetConfigResponse\x12/\n\x07success\x18\x01 \x01(\x0b\x32\x1c.qm.grpc.v2.GetConfigSuccessH\x00\x12+\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x1a.qm.grpc.v2.GetConfigErrorH\x00\x42\x10\n\x0eresponse_oneof\"\x89\x03\n\x1eJobServiceGetJobStatusResponse\x12\x63\n\x07success\x18\x01 \x01(\x0b\x32P.qm.grpc.v2.JobServiceGetJobStatusResponse.JobServiceGetJobStatusResponseSuccessH\x00\x12_\n\x05\x65rror\x18\x02 \x01(\x0b\x32N.qm.grpc.v2.JobServiceGetJobStatusResponse.JobServiceGetJobStatusResponseErrorH\x00\x1a\x36\n#JobServiceGetJobStatusResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1aW\n%JobServiceGetJobStatusResponseSuccess\x12.\n\x06status\x18\x01 \x01(\x0e\x32\x1e.qm.grpc.v2.JobExecutionStatusB\x10\n\x0eresponse_oneof\"\xa1\x02\n\"JobServicePushToInputStreamRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x13\n\x0bstream_name\x18\x02 \x01(\t\x12=\n\x0fint_stream_data\x18\x03 \x01(\x0b\x32\".qm.grpc.job_manager.IntStreamDataH\x00\x12\x41\n\x11\x66ixed_stream_data\x18\x04 \x01(\x0b\x32$.qm.grpc.job_manager.FixedStreamDataH\x00\x12?\n\x10\x62ool_stream_data\x18\x05 \x01(\x0b\x32#.qm.grpc.job_manager.BoolStreamDataH\x00\x42\x13\n\x11stream_data_oneof\"\xb6\x02\n\x19PushToInputStreamResponse\x12Y\n\x07success\x18\x01 \x01(\x0b\x32\x46.qm.grpc.v2.PushToInputStreamResponse.PushToInputStreamResponseSuccessH\x00\x12U\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x44.qm.grpc.v2.PushToInputStreamResponse.PushToInputStreamResponseErrorH\x00\x1a\"\n PushToInputStreamResponseSuccess\x1a\x31\n\x1ePushToInputStreamResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"+\n\x19GetJobResultSchemaRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"\x82\x04\n\x1aGetJobResultSchemaResponse\x12[\n\x07success\x18\x01 \x01(\x0b\x32H.qm.grpc.v2.GetJobResultSchemaResponse.GetJobResultSchemaResponseSuccessH\x00\x12W\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x46.qm.grpc.v2.GetJobResultSchemaResponse.GetJobResultSchemaResponseErrorH\x00\x1a\xe7\x01\n!GetJobResultSchemaResponseSuccess\x12\\\n\x05items\x18\x01 \x03(\x0b\x32M.qm.grpc.v2.GetJobResultSchemaResponse.GetJobResultSchemaResponseSuccess.Item\x1a\x64\n\x04Item\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0csimple_dtype\x18\x02 \x01(\t\x12\x11\n\tis_single\x18\x03 \x01(\x08\x12\x16\n\x0e\x65xpected_count\x18\x04 \x01(\x05\x12\r\n\x05shape\x18\x05 \x03(\x05\x1a\x32\n\x1fGetJobResultSchemaResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"Z\n\x1eGetJobNamedResultHeaderRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x13\n\x0boutput_name\x18\x02 \x01(\t\x12\x13\n\x0b\x66lat_format\x18\x03 \x01(\x08\"\xac\x01\n GetJobNamedResultsHeadersRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x44\n\x07outputs\x18\x02 \x03(\x0b\x32\x33.qm.grpc.v2.GetJobNamedResultsHeadersRequest.Output\x1a\x32\n\x06Output\x12\x13\n\x0boutput_name\x18\x01 \x01(\t\x12\x13\n\x0b\x66lat_format\x18\x02 \x01(\x08\"\xdc\x04\n!GetJobNamedResultsHeadersResponse\x12i\n\x07success\x18\x01 \x01(\x0b\x32V.qm.grpc.v2.GetJobNamedResultsHeadersResponse.GetJobNamedResultsHeadersResponseSuccessH\x00\x12\x65\n\x05\x65rror\x18\x02 \x01(\x0b\x32T.qm.grpc.v2.GetJobNamedResultsHeadersResponse.GetJobNamedResultsHeadersResponseErrorH\x00\x1a\x97\x02\n(GetJobNamedResultsHeadersResponseSuccess\x12t\n\x07headers\x18\x01 \x03(\x0b\x32\x63.qm.grpc.v2.GetJobNamedResultsHeadersResponse.GetJobNamedResultsHeadersResponseSuccess.OutputHeader\x1au\n\x0cOutputHeader\x12\x14\n\x0c\x63ount_so_far\x18\x01 \x01(\x05\x12\x15\n\rhas_data_loss\x18\x02 \x01(\x08\x12\x13\n\x0boutput_name\x18\x03 \x01(\t\x12\x14\n\x0csimple_dtype\x18\x04 \x01(\t\x12\r\n\x05shape\x18\x05 \x03(\x05\x1a\x39\n&GetJobNamedResultsHeadersResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof\"\xdb\x03\n\x1fGetJobNamedResultHeaderResponse\x12\x65\n\x07success\x18\x01 \x01(\x0b\x32R.qm.grpc.v2.GetJobNamedResultHeaderResponse.GetJobNamedResultHeaderResponseSuccessH\x00\x12\x61\n\x05\x65rror\x18\x02 \x01(\x0b\x32P.qm.grpc.v2.GetJobNamedResultHeaderResponse.GetJobNamedResultHeaderResponseErrorH\x00\x1a\x37\n$GetJobNamedResultHeaderResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\xa2\x01\n&GetJobNamedResultHeaderResponseSuccess\x12\x11\n\tis_single\x18\x01 \x01(\x08\x12\x14\n\x0c\x63ount_so_far\x18\x02 \x01(\x05\x12\x14\n\x0csimple_dtype\x18\x03 \x01(\t\x12\x15\n\rhas_data_loss\x18\x04 \x01(\x08\x12\r\n\x05shape\x18\x05 \x03(\x05\x12\x13\n\x0boutput_name\x18\x06 \x01(\tB\x10\n\x0eresponse_oneof\"\xa5\x01\n\x16GetNamedResultsRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12:\n\x07outputs\x18\x02 \x03(\x0b\x32).qm.grpc.v2.GetNamedResultsRequest.Output\x1a?\n\x06Output\x12\x13\n\x0boutput_name\x18\x01 \x01(\t\x12 \n\x05range\x18\x02 \x01(\x0b\x32\x11.qm.grpc.v2.Range\"}\n\x15GetNamedResultRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x13\n\x0boutput_name\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x30\n\x0blong_offset\x18\x05 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\"\xf4\x04\n\x16GetNamedResultResponse\x12S\n\x07success\x18\x01 \x01(\x0b\x32@.qm.grpc.v2.GetNamedResultResponse.GetNamedResultResponseSuccessH\x00\x12O\n\x05\x65rror\x18\x02 \x01(\x0b\x32>.qm.grpc.v2.GetNamedResultResponse.GetNamedResultResponseErrorH\x00\x1a.\n\x1bGetNamedResultResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\xf1\x02\n\x1dGetNamedResultResponseSuccess\x12\x1a\n\x0e\x63ount_of_items\x18\x01 \x01(\x05\x42\x02\x18\x01\x12\x10\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x42\x02\x18\x01\x12`\n\ndata_chunk\x18\x03 \x01(\x0b\x32J.qm.grpc.v2.GetNamedResultResponse.GetNamedResultResponseSuccess.DataChunkH\x00\x12\x64\n\x0c\x64\x61ta_summary\x18\x04 \x01(\x0b\x32L.qm.grpc.v2.GetNamedResultResponse.GetNamedResultResponseSuccess.DataSummaryH\x00\x12\x13\n\x0boutput_name\x18\x05 \x01(\t\x1a\x19\n\tDataChunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x1a\x1c\n\x0b\x44\x61taSummary\x12\r\n\x05\x63ount\x18\x01 \x01(\x05\x42\x0c\n\ndata_oneofB\x10\n\x0eresponse_oneof\"%\n\x13GetJobErrorsRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"\xb4\x04\n\x14GetJobErrorsResponse\x12O\n\x07success\x18\x01 \x01(\x0b\x32<.qm.grpc.v2.GetJobErrorsResponse.GetJobErrorsResponseSuccessH\x00\x12K\n\x05\x65rror\x18\x02 \x01(\x0b\x32:.qm.grpc.v2.GetJobErrorsResponse.GetJobErrorsResponseErrorH\x00\x1a,\n\x19GetJobErrorsResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\xbd\x02\n\x1bGetJobErrorsResponseSuccess\x12R\n\x06\x65rrors\x18\x01 \x03(\x0b\x32\x42.qm.grpc.v2.GetJobErrorsResponse.GetJobErrorsResponseSuccess.Error\x1a\x97\x01\n\x05\x45rror\x12\x11\n\terrorCode\x18\x01 \x01(\x05\x12j\n\rerrorSeverity\x18\x02 \x01(\x0e\x32S.qm.grpc.v2.GetJobErrorsResponse.GetJobErrorsResponseSuccess.ExecutionErrorSeverity\x12\x0f\n\x07message\x18\x03 \x01(\t\"0\n\x16\x45xecutionErrorSeverity\x12\t\n\x05\x45RROR\x10\x00\x12\x0b\n\x07WARNING\x10\x01\x42\x10\n\x0eresponse_oneof\"\x1f\n\rCancelRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"\xd5\x01\n\x0e\x43\x61ncelResponse\x12>\n\x07success\x18\x01 \x01(\x0b\x32+.qm.grpc.v2.CancelResponse.CancelJobSuccessH\x00\x12:\n\x05\x65rror\x18\x02 \x01(\x0b\x32).qm.grpc.v2.CancelResponse.CancelJobErrorH\x00\x1a!\n\x0e\x43\x61ncelJobError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\x12\n\x10\x43\x61ncelJobSuccessB\x10\n\x0eresponse_oneof\"*\n\x18GetJobResultStateRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"\xea\x02\n\x19GetJobResultStateResponse\x12Y\n\x07success\x18\x01 \x01(\x0b\x32\x46.qm.grpc.v2.GetJobResultStateResponse.GetJobResultStateResponseSuccessH\x00\x12U\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x44.qm.grpc.v2.GetJobResultStateResponse.GetJobResultStateResponseErrorH\x00\x1a\x31\n\x1eGetJobResultStateResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1aV\n GetJobResultStateResponseSuccess\x12\x0c\n\x04\x64one\x18\x01 \x01(\x08\x12\x0e\n\x06\x63losed\x18\x02 \x01(\x08\x12\x14\n\x0chas_dataloss\x18\x03 \x01(\x08\x42\x10\n\x0eresponse_oneof\"U\n\x12PullSamplesRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x16\n\x0einclude_analog\x18\x02 \x01(\x08\x12\x17\n\x0finclude_digital\x18\x03 \x01(\x08\"\xeb\x07\n\x13PullSamplesResponse\x12M\n\x07success\x18\x01 \x01(\x0b\x32:.qm.grpc.v2.PullSamplesResponse.PullSamplesResponseSuccessH\x00\x12I\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x38.qm.grpc.v2.PullSamplesResponse.PullSamplesResponseErrorH\x00\x1a+\n\x18PullSamplesResponseError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\xfa\x05\n\x1aPullSamplesResponseSuccess\x12\x12\n\ncontroller\x18\x01 \x01(\t\x12\x0e\n\x06\x66\x65m_id\x18\x02 \x01(\x05\x12\x0f\n\x07port_id\x18\x03 \x01(\x05\x12M\n\x04mode\x18\x04 \x01(\x0e\x32?.qm.grpc.v2.PullSamplesResponse.PullSamplesResponseSuccess.Mode\x12\\\n\x0b\x64ouble_data\x18\x05 \x01(\x0b\x32\x45.qm.grpc.v2.PullSamplesResponse.PullSamplesResponseSuccess.doubleDataH\x00\x12^\n\x0c\x62oolean_data\x18\x06 \x01(\x0b\x32\x46.qm.grpc.v2.PullSamplesResponse.PullSamplesResponseSuccess.booleanDataH\x00\x1a\xb0\x01\n\ndoubleData\x12K\n\x02mw\x18\x01 \x01(\x0b\x32=.qm.grpc.v2.PullSamplesResponse.PullSamplesResponseSuccess.mwH\x00\x12K\n\x02lf\x18\x02 \x01(\x0b\x32=.qm.grpc.v2.PullSamplesResponse.PullSamplesResponseSuccess.lfH\x00\x42\x08\n\x06output\x1a)\n\x02mw\x12\r\n\x05\x64ucId\x18\x01 \x01(\x05\x12\t\n\x01I\x18\x02 \x03(\x01\x12\t\n\x01Q\x18\x03 \x03(\x01\x1a\x13\n\x02lf\x12\r\n\x05items\x18\x01 \x03(\x01\x1a~\n\x0b\x62ooleanData\x12Y\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32K.qm.grpc.v2.PullSamplesResponse.PullSamplesResponseSuccess.booleanData.Data\x1a\x14\n\x04\x44\x61ta\x12\x0c\n\x04item\x18\x01 \x03(\x08\"\x1f\n\x04Mode\x12\n\n\x06\x41NALOG\x10\x00\x12\x0b\n\x07\x44IGITAL\x10\x01\x42\x06\n\x04\x64\x61taB\x10\n\x0eresponse_oneof\"\xfd\x01\n\x1dSetOscillatorFrequencyRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\n\n\x02qe\x18\x02 \x01(\t\x12\x18\n\x10new_frequency_hz\x18\x03 \x01(\x01\x12\\\n\x10update_component\x18\x04 \x01(\x0e\x32\x42.qm.grpc.v2.SetOscillatorFrequencyRequest.UpdateComponentSelection\"H\n\x18UpdateComponentSelection\x12\x08\n\x04\x62oth\x10\x00\x12\x0f\n\x0bupconverter\x10\x01\x12\x11\n\rdownconverter\x10\x02\"\xc5\x02\n\x1eSetOscillatorFrequencyResponse\x12^\n\x07success\x18\x01 \x01(\x0b\x32K.qm.grpc.v2.SetOscillatorFrequencyResponse.ChangeOscillatorFrequencySuccessH\x00\x12Z\n\x05\x65rror\x18\x02 \x01(\x0b\x32I.qm.grpc.v2.SetOscillatorFrequencyResponse.ChangeOscillatorFrequencyErrorH\x00\x1a\x31\n\x1e\x43hangeOscillatorFrequencyError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\t\x1a\"\n ChangeOscillatorFrequencySuccessB\x10\n\x0eresponse_oneof\"*\n\x18GetWaveformReportRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t\"\xc7\x02\n\x19GetWaveformReportResponse\x12Q\n\x07success\x18\x01 \x01(\x0b\x32>.qm.grpc.v2.GetWaveformReportResponse.GetWaveformReportSuccessH\x00\x12M\n\x05\x65rror\x18\x02 \x01(\x0b\x32<.qm.grpc.v2.GetWaveformReportResponse.GetWaveformReportErrorH\x00\x1aK\n\x18GetWaveformReportSuccess\x12/\n\x0ewaveformReport\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x1a)\n\x16GetWaveformReportError\x12\x0f\n\x07\x64\x65tails\x18\x01 \x01(\tB\x10\n\x0eresponse_oneof2\xf6\x17\n\nJobService\x12^\n\tGetConfig\x12&.qm.grpc.v2.JobServiceGetConfigRequest\x1a\'.qm.grpc.v2.JobServiceGetConfigResponse\"\x00\x12g\n\x0cGetJobStatus\x12).qm.grpc.v2.JobServiceGetJobStatusRequest\x1a*.qm.grpc.v2.JobServiceGetJobStatusResponse\"\x00\x12p\n\x13GetJobStatusUpdates\x12).qm.grpc.v2.JobServiceGetJobStatusRequest\x1a*.qm.grpc.v2.JobServiceGetJobStatusResponse\"\x00\x30\x01\x12l\n\x11PushToInputStream\x12..qm.grpc.v2.JobServicePushToInputStreamRequest\x1a%.qm.grpc.v2.PushToInputStreamResponse\"\x00\x12Q\n\x08IsPaused\x12%.qm.grpc.v2.JobServiceIsPausedRequest\x1a\x1c.qm.grpc.v2.IsPausedResponse\"\x00\x12\x41\n\x06Resume\x12\x19.qm.grpc.v2.ResumeRequest\x1a\x1a.qm.grpc.v2.ResumeResponse\"\x00\x12h\n\x13SetMatrixCorrection\x12&.qm.grpc.v2.SetMatrixCorrectionRequest\x1a\'.qm.grpc.v2.SetMatrixCorrectionResponse\"\x00\x12h\n\x13GetMatrixCorrection\x12&.qm.grpc.v2.GetMatrixCorrectionRequest\x1a\'.qm.grpc.v2.GetMatrixCorrectionResponse\"\x00\x12w\n\x18SetIntermediateFrequency\x12+.qm.grpc.v2.SetIntermediateFrequencyRequest\x1a,.qm.grpc.v2.SetIntermediateFrequencyResponse\"\x00\x12w\n\x18GetIntermediateFrequency\x12+.qm.grpc.v2.GetIntermediateFrequencyRequest\x1a,.qm.grpc.v2.GetIntermediateFrequencyResponse\"\x00\x12\\\n\x0fSetDigitalDelay\x12\".qm.grpc.v2.SetDigitalDelayRequest\x1a#.qm.grpc.v2.SetDigitalDelayResponse\"\x00\x12\\\n\x0fGetDigitalDelay\x12\".qm.grpc.v2.GetDigitalDelayRequest\x1a#.qm.grpc.v2.GetDigitalDelayResponse\"\x00\x12_\n\x10SetDigitalBuffer\x12#.qm.grpc.v2.SetDigitalBufferRequest\x1a$.qm.grpc.v2.SetDigitalBufferResponse\"\x00\x12_\n\x10GetDigitalBuffer\x12#.qm.grpc.v2.GetDigitalBufferRequest\x1a$.qm.grpc.v2.GetDigitalBufferResponse\"\x00\x12_\n\x10SetInputDcOffset\x12#.qm.grpc.v2.SetInputDcOffsetRequest\x1a$.qm.grpc.v2.SetInputDcOffsetResponse\"\x00\x12_\n\x10GetInputDcOffset\x12#.qm.grpc.v2.GetInputDcOffsetRequest\x1a$.qm.grpc.v2.GetInputDcOffsetResponse\"\x00\x12\x62\n\x11SetOutputDcOffset\x12$.qm.grpc.v2.SetOutputDcOffsetRequest\x1a%.qm.grpc.v2.SetOutputDcOffsetResponse\"\x00\x12\x62\n\x11GetOutputDcOffset\x12$.qm.grpc.v2.GetOutputDcOffsetRequest\x1a%.qm.grpc.v2.GetOutputDcOffsetResponse\"\x00\x12q\n\x16SetOscillatorFrequency\x12).qm.grpc.v2.SetOscillatorFrequencyRequest\x1a*.qm.grpc.v2.SetOscillatorFrequencyResponse\"\x00\x12P\n\x0bSetIOValues\x12\x1e.qm.grpc.v2.SetIoValuesRequest\x1a\x1f.qm.grpc.v2.SetIoValuesResponse\"\x00\x12P\n\x0bGetIOValues\x12\x1e.qm.grpc.v2.GetIoValuesRequest\x1a\x1f.qm.grpc.v2.GetIoValuesResponse\"\x00\x12\x65\n\x12GetJobResultSchema\x12%.qm.grpc.v2.GetJobResultSchemaRequest\x1a&.qm.grpc.v2.GetJobResultSchemaResponse\"\x00\x12t\n\x17GetJobNamedResultHeader\x12*.qm.grpc.v2.GetJobNamedResultHeaderRequest\x1a+.qm.grpc.v2.GetJobNamedResultHeaderResponse\"\x00\x12z\n\x19GetJobNamedResultsHeaders\x12,.qm.grpc.v2.GetJobNamedResultsHeadersRequest\x1a-.qm.grpc.v2.GetJobNamedResultsHeadersResponse\"\x00\x12[\n\x0eGetNamedResult\x12!.qm.grpc.v2.GetNamedResultRequest\x1a\".qm.grpc.v2.GetNamedResultResponse\"\x00\x30\x01\x12]\n\x0fGetNamedResults\x12\".qm.grpc.v2.GetNamedResultsRequest\x1a\".qm.grpc.v2.GetNamedResultResponse\"\x00\x30\x01\x12P\n\tGetErrors\x12\x1f.qm.grpc.v2.GetJobErrorsRequest\x1a .qm.grpc.v2.GetJobErrorsResponse\"\x00\x12\x62\n\x11GetJobResultState\x12$.qm.grpc.v2.GetJobResultStateRequest\x1a%.qm.grpc.v2.GetJobResultStateResponse\"\x00\x12R\n\x0bPullSamples\x12\x1e.qm.grpc.v2.PullSamplesRequest\x1a\x1f.qm.grpc.v2.PullSamplesResponse\"\x00\x30\x01\x12\x41\n\x06\x43\x61ncel\x12\x19.qm.grpc.v2.CancelRequest\x1a\x1a.qm.grpc.v2.CancelResponse\"\x00\x12\x62\n\x11GetWaveformReport\x12$.qm.grpc.v2.GetWaveformReportRequest\x1a%.qm.grpc.v2.GetWaveformReportResponse\"\x00\x42\x1a\n\x10qm.client.api.v2P\x01\xa2\x02\x03HLWb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qm.grpc.v2.job_api_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\020qm.client.api.v2P\001\242\002\003HLW'
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS'].fields_by_name['count_of_items']._loaded_options = None
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS'].fields_by_name['count_of_items']._serialized_options = b'\030\001'
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS'].fields_by_name['data']._loaded_options = None
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS'].fields_by_name['data']._serialized_options = b'\030\001'
  _globals['_SETMATRIXCORRECTIONRESPONSE']._serialized_start=233
  _globals['_SETMATRIXCORRECTIONRESPONSE']._serialized_end=557
  _globals['_SETMATRIXCORRECTIONRESPONSE_SETMATRIXCORRECTIONRESPONSEERROR']._serialized_start=450
  _globals['_SETMATRIXCORRECTIONRESPONSE_SETMATRIXCORRECTIONRESPONSEERROR']._serialized_end=501
  _globals['_SETMATRIXCORRECTIONRESPONSE_SETMATRIXCORRECTIONRESPONSESUCCESS']._serialized_start=503
  _globals['_SETMATRIXCORRECTIONRESPONSE_SETMATRIXCORRECTIONRESPONSESUCCESS']._serialized_end=539
  _globals['_SETINTERMEDIATEFREQUENCYRESPONSE']._serialized_start=560
  _globals['_SETINTERMEDIATEFREQUENCYRESPONSE']._serialized_end=919
  _globals['_SETINTERMEDIATEFREQUENCYRESPONSE_SETINTERMEDIATEFREQUENCYRESPONSEERROR']._serialized_start=802
  _globals['_SETINTERMEDIATEFREQUENCYRESPONSE_SETINTERMEDIATEFREQUENCYRESPONSEERROR']._serialized_end=858
  _globals['_SETINTERMEDIATEFREQUENCYRESPONSE_SETINTERMEDIATEFREQUENCYRESPONSESUCCESS']._serialized_start=860
  _globals['_SETINTERMEDIATEFREQUENCYRESPONSE_SETINTERMEDIATEFREQUENCYRESPONSESUCCESS']._serialized_end=901
  _globals['_SETDIGITALDELAYRESPONSE']._serialized_start=922
  _globals['_SETDIGITALDELAYRESPONSE']._serialized_end=1218
  _globals['_SETDIGITALDELAYRESPONSE_SETDIGITALDELAYRESPONSEERROR']._serialized_start=1119
  _globals['_SETDIGITALDELAYRESPONSE_SETDIGITALDELAYRESPONSEERROR']._serialized_end=1166
  _globals['_SETDIGITALDELAYRESPONSE_SETDIGITALDELAYRESPONSESUCCESS']._serialized_start=1168
  _globals['_SETDIGITALDELAYRESPONSE_SETDIGITALDELAYRESPONSESUCCESS']._serialized_end=1200
  _globals['_SETDIGITALBUFFERRESPONSE']._serialized_start=1221
  _globals['_SETDIGITALBUFFERRESPONSE']._serialized_end=1524
  _globals['_SETDIGITALBUFFERRESPONSE_SETDIGITALBUFFERRESPONSEERROR']._serialized_start=1423
  _globals['_SETDIGITALBUFFERRESPONSE_SETDIGITALBUFFERRESPONSEERROR']._serialized_end=1471
  _globals['_SETDIGITALBUFFERRESPONSE_SETDIGITALBUFFERRESPONSESUCCESS']._serialized_start=1473
  _globals['_SETDIGITALBUFFERRESPONSE_SETDIGITALBUFFERRESPONSESUCCESS']._serialized_end=1506
  _globals['_SETINPUTDCOFFSETRESPONSE']._serialized_start=1527
  _globals['_SETINPUTDCOFFSETRESPONSE']._serialized_end=1830
  _globals['_SETINPUTDCOFFSETRESPONSE_SETINPUTDCOFFSETRESPONSEERROR']._serialized_start=1729
  _globals['_SETINPUTDCOFFSETRESPONSE_SETINPUTDCOFFSETRESPONSEERROR']._serialized_end=1777
  _globals['_SETINPUTDCOFFSETRESPONSE_SETINPUTDCOFFSETRESPONSESUCCESS']._serialized_start=1779
  _globals['_SETINPUTDCOFFSETRESPONSE_SETINPUTDCOFFSETRESPONSESUCCESS']._serialized_end=1812
  _globals['_SETOUTPUTDCOFFSETRESPONSE']._serialized_start=1833
  _globals['_SETOUTPUTDCOFFSETRESPONSE']._serialized_end=2143
  _globals['_SETOUTPUTDCOFFSETRESPONSE_SETOUTPUTDCOFFSETRESPONSEERROR']._serialized_start=2040
  _globals['_SETOUTPUTDCOFFSETRESPONSE_SETOUTPUTDCOFFSETRESPONSEERROR']._serialized_end=2089
  _globals['_SETOUTPUTDCOFFSETRESPONSE_SETOUTPUTDCOFFSETRESPONSESUCCESS']._serialized_start=2091
  _globals['_SETOUTPUTDCOFFSETRESPONSE_SETOUTPUTDCOFFSETRESPONSESUCCESS']._serialized_end=2125
  _globals['_SETIOVALUESRESPONSE']._serialized_start=2146
  _globals['_SETIOVALUESRESPONSE']._serialized_end=2414
  _globals['_SETIOVALUESRESPONSE_SETIOVALUESRESPONSEERROR']._serialized_start=2323
  _globals['_SETIOVALUESRESPONSE_SETIOVALUESRESPONSEERROR']._serialized_end=2366
  _globals['_SETIOVALUESRESPONSE_SETIOVALUESRESPONSESUCCESS']._serialized_start=2368
  _globals['_SETIOVALUESRESPONSE_SETIOVALUESRESPONSESUCCESS']._serialized_end=2396
  _globals['_SETIOVALUESREQUEST']._serialized_start=2417
  _globals['_SETIOVALUESREQUEST']._serialized_end=2678
  _globals['_SETIOVALUESREQUEST_IOVALUESETDATA']._serialized_start=2575
  _globals['_SETIOVALUESREQUEST_IOVALUESETDATA']._serialized_end=2678
  _globals['_GETIOVALUESREQUEST']._serialized_start=2680
  _globals['_GETIOVALUESREQUEST']._serialized_end=2716
  _globals['_IOVALUESDATA']._serialized_start=2718
  _globals['_IOVALUESDATA']._serialized_end=2796
  _globals['_GETIOVALUESRESPONSE']._serialized_start=2799
  _globals['_GETIOVALUESRESPONSE']._serialized_end=3320
  _globals['_GETIOVALUESRESPONSE_GETIOVALUESRESPONSESUCCESS']._serialized_start=2977
  _globals['_GETIOVALUESRESPONSE_GETIOVALUESRESPONSESUCCESS']._serialized_end=3257
  _globals['_GETIOVALUESRESPONSE_GETIOVALUESRESPONSESUCCESS_IOVALUESDATA']._serialized_start=2718
  _globals['_GETIOVALUESRESPONSE_GETIOVALUESRESPONSESUCCESS_IOVALUESDATA']._serialized_end=2796
  _globals['_GETIOVALUESRESPONSE_GETIOVALUESRESPONSEERROR']._serialized_start=3259
  _globals['_GETIOVALUESRESPONSE_GETIOVALUESRESPONSEERROR']._serialized_end=3302
  _globals['_JOBSERVICEISPAUSEDREQUEST']._serialized_start=3322
  _globals['_JOBSERVICEISPAUSEDREQUEST']._serialized_end=3365
  _globals['_ISPAUSEDRESPONSE']._serialized_start=3368
  _globals['_ISPAUSEDRESPONSE']._serialized_end=3634
  _globals['_ISPAUSEDRESPONSE_ISPAUSEDRESPONSEERROR']._serialized_start=3530
  _globals['_ISPAUSEDRESPONSE_ISPAUSEDRESPONSEERROR']._serialized_end=3570
  _globals['_ISPAUSEDRESPONSE_ISPAUSEDRESPONSESUCCESS']._serialized_start=3572
  _globals['_ISPAUSEDRESPONSE_ISPAUSEDRESPONSESUCCESS']._serialized_end=3616
  _globals['_JOBSERVICEGETCONFIGREQUEST']._serialized_start=3636
  _globals['_JOBSERVICEGETCONFIGREQUEST']._serialized_end=3680
  _globals['_JOBSERVICEGETJOBSTATUSREQUEST']._serialized_start=3682
  _globals['_JOBSERVICEGETJOBSTATUSREQUEST']._serialized_end=3729
  _globals['_RESUMEREQUEST']._serialized_start=3731
  _globals['_RESUMEREQUEST']._serialized_end=3762
  _globals['_RESUMERESPONSE']._serialized_start=3765
  _globals['_RESUMERESPONSE']._serialized_end=3998
  _globals['_RESUMERESPONSE_RESUMERESPONSEERROR']._serialized_start=3917
  _globals['_RESUMERESPONSE_RESUMERESPONSEERROR']._serialized_end=3955
  _globals['_RESUMERESPONSE_RESUMERESPONSESUCCESS']._serialized_start=3957
  _globals['_RESUMERESPONSE_RESUMERESPONSESUCCESS']._serialized_end=3980
  _globals['_SETMATRIXCORRECTIONREQUEST']._serialized_start=4000
  _globals['_SETMATRIXCORRECTIONREQUEST']._serialized_end=4100
  _globals['_SETINTERMEDIATEFREQUENCYREQUEST']._serialized_start=4102
  _globals['_SETINTERMEDIATEFREQUENCYREQUEST']._serialized_end=4182
  _globals['_SETDIGITALDELAYREQUEST']._serialized_start=4184
  _globals['_SETDIGITALDELAYREQUEST']._serialized_end=4265
  _globals['_SETDIGITALBUFFERREQUEST']._serialized_start=4267
  _globals['_SETDIGITALBUFFERREQUEST']._serialized_end=4350
  _globals['_SETINPUTDCOFFSETREQUEST']._serialized_start=4352
  _globals['_SETINPUTDCOFFSETREQUEST']._serialized_end=4435
  _globals['_SETOUTPUTDCOFFSETREQUEST']._serialized_start=4438
  _globals['_SETOUTPUTDCOFFSETREQUEST']._serialized_end=4630
  _globals['_GETMATRIXCORRECTIONREQUEST']._serialized_start=4632
  _globals['_GETMATRIXCORRECTIONREQUEST']._serialized_end=4688
  _globals['_GETMATRIXCORRECTIONERROR']._serialized_start=4690
  _globals['_GETMATRIXCORRECTIONERROR']._serialized_end=4733
  _globals['_GETMATRIXCORRECTIONRESPONSE']._serialized_start=4736
  _globals['_GETMATRIXCORRECTIONRESPONSE']._serialized_end=5140
  _globals['_GETMATRIXCORRECTIONRESPONSE_GETMATRIXCORRECTIONRESPONSESUCCESS']._serialized_start=4953
  _globals['_GETMATRIXCORRECTIONRESPONSE_GETMATRIXCORRECTIONRESPONSESUCCESS']._serialized_end=5033
  _globals['_GETMATRIXCORRECTIONRESPONSE_GETMATRIXCORRECTIONRESPONSEERROR']._serialized_start=5035
  _globals['_GETMATRIXCORRECTIONRESPONSE_GETMATRIXCORRECTIONRESPONSEERROR']._serialized_end=5122
  _globals['_GETINTERMEDIATEFREQUENCYREQUEST']._serialized_start=5142
  _globals['_GETINTERMEDIATEFREQUENCYREQUEST']._serialized_end=5203
  _globals['_GETINTERMEDIATEFREQUENCYRESPONSE']._serialized_start=5206
  _globals['_GETINTERMEDIATEFREQUENCYRESPONSE']._serialized_end=5584
  _globals['_GETINTERMEDIATEFREQUENCYRESPONSE_GETINTERMEDIATEFREQUENCYRESPONSEERROR']._serialized_start=5448
  _globals['_GETINTERMEDIATEFREQUENCYRESPONSE_GETINTERMEDIATEFREQUENCYRESPONSEERROR']._serialized_end=5504
  _globals['_GETINTERMEDIATEFREQUENCYRESPONSE_GETINTERMEDIATEFREQUENCYRESPONSESUCCESS']._serialized_start=5506
  _globals['_GETINTERMEDIATEFREQUENCYRESPONSE_GETINTERMEDIATEFREQUENCYRESPONSESUCCESS']._serialized_end=5566
  _globals['_GETDIGITALDELAYREQUEST']._serialized_start=5586
  _globals['_GETDIGITALDELAYREQUEST']._serialized_end=5652
  _globals['_GETDIGITALDELAYRESPONSE']._serialized_start=5655
  _globals['_GETDIGITALDELAYRESPONSE']._serialized_end=5966
  _globals['_GETDIGITALDELAYRESPONSE_GETDIGITALDELAYRESPONSEERROR']._serialized_start=5852
  _globals['_GETDIGITALDELAYRESPONSE_GETDIGITALDELAYRESPONSEERROR']._serialized_end=5899
  _globals['_GETDIGITALDELAYRESPONSE_GETDIGITALDELAYRESPONSESUCCESS']._serialized_start=5901
  _globals['_GETDIGITALDELAYRESPONSE_GETDIGITALDELAYRESPONSESUCCESS']._serialized_end=5948
  _globals['_GETDIGITALBUFFERREQUEST']._serialized_start=5968
  _globals['_GETDIGITALBUFFERREQUEST']._serialized_end=6035
  _globals['_GETDIGITALBUFFERRESPONSE']._serialized_start=6038
  _globals['_GETDIGITALBUFFERRESPONSE']._serialized_end=6357
  _globals['_GETDIGITALBUFFERRESPONSE_GETDIGITALBUFFERRESPONSEERROR']._serialized_start=6240
  _globals['_GETDIGITALBUFFERRESPONSE_GETDIGITALBUFFERRESPONSEERROR']._serialized_end=6288
  _globals['_GETDIGITALBUFFERRESPONSE_GETDIGITALBUFFERRESPONSESUCCESS']._serialized_start=6290
  _globals['_GETDIGITALBUFFERRESPONSE_GETDIGITALBUFFERRESPONSESUCCESS']._serialized_end=6339
  _globals['_GETINPUTDCOFFSETREQUEST']._serialized_start=6359
  _globals['_GETINPUTDCOFFSETREQUEST']._serialized_end=6426
  _globals['_GETINPUTDCOFFSETERROR']._serialized_start=6428
  _globals['_GETINPUTDCOFFSETERROR']._serialized_end=6468
  _globals['_GETINPUTDCOFFSETSUCCESS']._serialized_start=6470
  _globals['_GETINPUTDCOFFSETSUCCESS']._serialized_end=6511
  _globals['_GETINPUTDCOFFSETRESPONSE']._serialized_start=6514
  _globals['_GETINPUTDCOFFSETRESPONSE']._serialized_end=6833
  _globals['_GETINPUTDCOFFSETRESPONSE_GETINPUTDCOFFSETRESPONSEERROR']._serialized_start=6716
  _globals['_GETINPUTDCOFFSETRESPONSE_GETINPUTDCOFFSETRESPONSEERROR']._serialized_end=6764
  _globals['_GETINPUTDCOFFSETRESPONSE_GETINPUTDCOFFSETRESPONSESUCCESS']._serialized_start=6766
  _globals['_GETINPUTDCOFFSETRESPONSE_GETINPUTDCOFFSETRESPONSESUCCESS']._serialized_end=6815
  _globals['_GETOUTPUTDCOFFSETREQUEST']._serialized_start=6835
  _globals['_GETOUTPUTDCOFFSETREQUEST']._serialized_end=6889
  _globals['_GETOUTPUTDCOFFSETRESPONSE']._serialized_start=6892
  _globals['_GETOUTPUTDCOFFSETRESPONSE']._serialized_end=7341
  _globals['_GETOUTPUTDCOFFSETRESPONSE_GETOUTPUTDCOFFSETRESPONSEERROR']._serialized_start=7099
  _globals['_GETOUTPUTDCOFFSETRESPONSE_GETOUTPUTDCOFFSETRESPONSEERROR']._serialized_end=7148
  _globals['_GETOUTPUTDCOFFSETRESPONSE_GETOUTPUTDCOFFSETRESPONSESUCCESS']._serialized_start=7151
  _globals['_GETOUTPUTDCOFFSETRESPONSE_GETOUTPUTDCOFFSETRESPONSESUCCESS']._serialized_end=7323
  _globals['_MIXINPUTSDCOFFSET']._serialized_start=7343
  _globals['_MIXINPUTSDCOFFSET']._serialized_end=7406
  _globals['_SINGLEINPUTDCOFFSET']._serialized_start=7408
  _globals['_SINGLEINPUTDCOFFSET']._serialized_end=7445
  _globals['_JOBSERVICEGETCONFIGRESPONSE']._serialized_start=7448
  _globals['_JOBSERVICEGETCONFIGRESPONSE']._serialized_end=7589
  _globals['_JOBSERVICEGETJOBSTATUSRESPONSE']._serialized_start=7592
  _globals['_JOBSERVICEGETJOBSTATUSRESPONSE']._serialized_end=7985
  _globals['_JOBSERVICEGETJOBSTATUSRESPONSE_JOBSERVICEGETJOBSTATUSRESPONSEERROR']._serialized_start=7824
  _globals['_JOBSERVICEGETJOBSTATUSRESPONSE_JOBSERVICEGETJOBSTATUSRESPONSEERROR']._serialized_end=7878
  _globals['_JOBSERVICEGETJOBSTATUSRESPONSE_JOBSERVICEGETJOBSTATUSRESPONSESUCCESS']._serialized_start=7880
  _globals['_JOBSERVICEGETJOBSTATUSRESPONSE_JOBSERVICEGETJOBSTATUSRESPONSESUCCESS']._serialized_end=7967
  _globals['_JOBSERVICEPUSHTOINPUTSTREAMREQUEST']._serialized_start=7988
  _globals['_JOBSERVICEPUSHTOINPUTSTREAMREQUEST']._serialized_end=8277
  _globals['_PUSHTOINPUTSTREAMRESPONSE']._serialized_start=8280
  _globals['_PUSHTOINPUTSTREAMRESPONSE']._serialized_end=8590
  _globals['_PUSHTOINPUTSTREAMRESPONSE_PUSHTOINPUTSTREAMRESPONSESUCCESS']._serialized_start=8487
  _globals['_PUSHTOINPUTSTREAMRESPONSE_PUSHTOINPUTSTREAMRESPONSESUCCESS']._serialized_end=8521
  _globals['_PUSHTOINPUTSTREAMRESPONSE_PUSHTOINPUTSTREAMRESPONSEERROR']._serialized_start=8523
  _globals['_PUSHTOINPUTSTREAMRESPONSE_PUSHTOINPUTSTREAMRESPONSEERROR']._serialized_end=8572
  _globals['_GETJOBRESULTSCHEMAREQUEST']._serialized_start=8592
  _globals['_GETJOBRESULTSCHEMAREQUEST']._serialized_end=8635
  _globals['_GETJOBRESULTSCHEMARESPONSE']._serialized_start=8638
  _globals['_GETJOBRESULTSCHEMARESPONSE']._serialized_end=9152
  _globals['_GETJOBRESULTSCHEMARESPONSE_GETJOBRESULTSCHEMARESPONSESUCCESS']._serialized_start=8851
  _globals['_GETJOBRESULTSCHEMARESPONSE_GETJOBRESULTSCHEMARESPONSESUCCESS']._serialized_end=9082
  _globals['_GETJOBRESULTSCHEMARESPONSE_GETJOBRESULTSCHEMARESPONSESUCCESS_ITEM']._serialized_start=8982
  _globals['_GETJOBRESULTSCHEMARESPONSE_GETJOBRESULTSCHEMARESPONSESUCCESS_ITEM']._serialized_end=9082
  _globals['_GETJOBRESULTSCHEMARESPONSE_GETJOBRESULTSCHEMARESPONSEERROR']._serialized_start=9084
  _globals['_GETJOBRESULTSCHEMARESPONSE_GETJOBRESULTSCHEMARESPONSEERROR']._serialized_end=9134
  _globals['_GETJOBNAMEDRESULTHEADERREQUEST']._serialized_start=9154
  _globals['_GETJOBNAMEDRESULTHEADERREQUEST']._serialized_end=9244
  _globals['_GETJOBNAMEDRESULTSHEADERSREQUEST']._serialized_start=9247
  _globals['_GETJOBNAMEDRESULTSHEADERSREQUEST']._serialized_end=9419
  _globals['_GETJOBNAMEDRESULTSHEADERSREQUEST_OUTPUT']._serialized_start=9369
  _globals['_GETJOBNAMEDRESULTSHEADERSREQUEST_OUTPUT']._serialized_end=9419
  _globals['_GETJOBNAMEDRESULTSHEADERSRESPONSE']._serialized_start=9422
  _globals['_GETJOBNAMEDRESULTSHEADERSRESPONSE']._serialized_end=10026
  _globals['_GETJOBNAMEDRESULTSHEADERSRESPONSE_GETJOBNAMEDRESULTSHEADERSRESPONSESUCCESS']._serialized_start=9670
  _globals['_GETJOBNAMEDRESULTSHEADERSRESPONSE_GETJOBNAMEDRESULTSHEADERSRESPONSESUCCESS']._serialized_end=9949
  _globals['_GETJOBNAMEDRESULTSHEADERSRESPONSE_GETJOBNAMEDRESULTSHEADERSRESPONSESUCCESS_OUTPUTHEADER']._serialized_start=9832
  _globals['_GETJOBNAMEDRESULTSHEADERSRESPONSE_GETJOBNAMEDRESULTSHEADERSRESPONSESUCCESS_OUTPUTHEADER']._serialized_end=9949
  _globals['_GETJOBNAMEDRESULTSHEADERSRESPONSE_GETJOBNAMEDRESULTSHEADERSRESPONSEERROR']._serialized_start=9951
  _globals['_GETJOBNAMEDRESULTSHEADERSRESPONSE_GETJOBNAMEDRESULTSHEADERSRESPONSEERROR']._serialized_end=10008
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE']._serialized_start=10029
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE']._serialized_end=10504
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE_GETJOBNAMEDRESULTHEADERRESPONSEERROR']._serialized_start=10266
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE_GETJOBNAMEDRESULTHEADERRESPONSEERROR']._serialized_end=10321
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE_GETJOBNAMEDRESULTHEADERRESPONSESUCCESS']._serialized_start=10324
  _globals['_GETJOBNAMEDRESULTHEADERRESPONSE_GETJOBNAMEDRESULTHEADERRESPONSESUCCESS']._serialized_end=10486
  _globals['_GETNAMEDRESULTSREQUEST']._serialized_start=10507
  _globals['_GETNAMEDRESULTSREQUEST']._serialized_end=10672
  _globals['_GETNAMEDRESULTSREQUEST_OUTPUT']._serialized_start=10609
  _globals['_GETNAMEDRESULTSREQUEST_OUTPUT']._serialized_end=10672
  _globals['_GETNAMEDRESULTREQUEST']._serialized_start=10674
  _globals['_GETNAMEDRESULTREQUEST']._serialized_end=10799
  _globals['_GETNAMEDRESULTRESPONSE']._serialized_start=10802
  _globals['_GETNAMEDRESULTRESPONSE']._serialized_end=11430
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSEERROR']._serialized_start=10994
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSEERROR']._serialized_end=11040
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS']._serialized_start=11043
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS']._serialized_end=11412
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS_DATACHUNK']._serialized_start=11343
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS_DATACHUNK']._serialized_end=11368
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS_DATASUMMARY']._serialized_start=11370
  _globals['_GETNAMEDRESULTRESPONSE_GETNAMEDRESULTRESPONSESUCCESS_DATASUMMARY']._serialized_end=11398
  _globals['_GETJOBERRORSREQUEST']._serialized_start=11432
  _globals['_GETJOBERRORSREQUEST']._serialized_end=11469
  _globals['_GETJOBERRORSRESPONSE']._serialized_start=11472
  _globals['_GETJOBERRORSRESPONSE']._serialized_end=12036
  _globals['_GETJOBERRORSRESPONSE_GETJOBERRORSRESPONSEERROR']._serialized_start=11654
  _globals['_GETJOBERRORSRESPONSE_GETJOBERRORSRESPONSEERROR']._serialized_end=11698
  _globals['_GETJOBERRORSRESPONSE_GETJOBERRORSRESPONSESUCCESS']._serialized_start=11701
  _globals['_GETJOBERRORSRESPONSE_GETJOBERRORSRESPONSESUCCESS']._serialized_end=12018
  _globals['_GETJOBERRORSRESPONSE_GETJOBERRORSRESPONSESUCCESS_ERROR']._serialized_start=11817
  _globals['_GETJOBERRORSRESPONSE_GETJOBERRORSRESPONSESUCCESS_ERROR']._serialized_end=11968
  _globals['_GETJOBERRORSRESPONSE_GETJOBERRORSRESPONSESUCCESS_EXECUTIONERRORSEVERITY']._serialized_start=11970
  _globals['_GETJOBERRORSRESPONSE_GETJOBERRORSRESPONSESUCCESS_EXECUTIONERRORSEVERITY']._serialized_end=12018
  _globals['_CANCELREQUEST']._serialized_start=12038
  _globals['_CANCELREQUEST']._serialized_end=12069
  _globals['_CANCELRESPONSE']._serialized_start=12072
  _globals['_CANCELRESPONSE']._serialized_end=12285
  _globals['_CANCELRESPONSE_CANCELJOBERROR']._serialized_start=12214
  _globals['_CANCELRESPONSE_CANCELJOBERROR']._serialized_end=12247
  _globals['_CANCELRESPONSE_CANCELJOBSUCCESS']._serialized_start=12249
  _globals['_CANCELRESPONSE_CANCELJOBSUCCESS']._serialized_end=12267
  _globals['_GETJOBRESULTSTATEREQUEST']._serialized_start=12287
  _globals['_GETJOBRESULTSTATEREQUEST']._serialized_end=12329
  _globals['_GETJOBRESULTSTATERESPONSE']._serialized_start=12332
  _globals['_GETJOBRESULTSTATERESPONSE']._serialized_end=12694
  _globals['_GETJOBRESULTSTATERESPONSE_GETJOBRESULTSTATERESPONSEERROR']._serialized_start=12539
  _globals['_GETJOBRESULTSTATERESPONSE_GETJOBRESULTSTATERESPONSEERROR']._serialized_end=12588
  _globals['_GETJOBRESULTSTATERESPONSE_GETJOBRESULTSTATERESPONSESUCCESS']._serialized_start=12590
  _globals['_GETJOBRESULTSTATERESPONSE_GETJOBRESULTSTATERESPONSESUCCESS']._serialized_end=12676
  _globals['_PULLSAMPLESREQUEST']._serialized_start=12696
  _globals['_PULLSAMPLESREQUEST']._serialized_end=12781
  _globals['_PULLSAMPLESRESPONSE']._serialized_start=12784
  _globals['_PULLSAMPLESRESPONSE']._serialized_end=13787
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSEERROR']._serialized_start=12961
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSEERROR']._serialized_end=13004
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS']._serialized_start=13007
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS']._serialized_end=13769
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_DOUBLEDATA']._serialized_start=13360
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_DOUBLEDATA']._serialized_end=13536
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_MW']._serialized_start=13538
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_MW']._serialized_end=13579
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_LF']._serialized_start=13581
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_LF']._serialized_end=13600
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_BOOLEANDATA']._serialized_start=13602
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_BOOLEANDATA']._serialized_end=13728
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_BOOLEANDATA_DATA']._serialized_start=13708
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_BOOLEANDATA_DATA']._serialized_end=13728
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_MODE']._serialized_start=13730
  _globals['_PULLSAMPLESRESPONSE_PULLSAMPLESRESPONSESUCCESS_MODE']._serialized_end=13761
  _globals['_SETOSCILLATORFREQUENCYREQUEST']._serialized_start=13790
  _globals['_SETOSCILLATORFREQUENCYREQUEST']._serialized_end=14043
  _globals['_SETOSCILLATORFREQUENCYREQUEST_UPDATECOMPONENTSELECTION']._serialized_start=13971
  _globals['_SETOSCILLATORFREQUENCYREQUEST_UPDATECOMPONENTSELECTION']._serialized_end=14043
  _globals['_SETOSCILLATORFREQUENCYRESPONSE']._serialized_start=14046
  _globals['_SETOSCILLATORFREQUENCYRESPONSE']._serialized_end=14371
  _globals['_SETOSCILLATORFREQUENCYRESPONSE_CHANGEOSCILLATORFREQUENCYERROR']._serialized_start=14268
  _globals['_SETOSCILLATORFREQUENCYRESPONSE_CHANGEOSCILLATORFREQUENCYERROR']._serialized_end=14317
  _globals['_SETOSCILLATORFREQUENCYRESPONSE_CHANGEOSCILLATORFREQUENCYSUCCESS']._serialized_start=14319
  _globals['_SETOSCILLATORFREQUENCYRESPONSE_CHANGEOSCILLATORFREQUENCYSUCCESS']._serialized_end=14353
  _globals['_GETWAVEFORMREPORTREQUEST']._serialized_start=14373
  _globals['_GETWAVEFORMREPORTREQUEST']._serialized_end=14415
  _globals['_GETWAVEFORMREPORTRESPONSE']._serialized_start=14418
  _globals['_GETWAVEFORMREPORTRESPONSE']._serialized_end=14745
  _globals['_GETWAVEFORMREPORTRESPONSE_GETWAVEFORMREPORTSUCCESS']._serialized_start=14609
  _globals['_GETWAVEFORMREPORTRESPONSE_GETWAVEFORMREPORTSUCCESS']._serialized_end=14684
  _globals['_GETWAVEFORMREPORTRESPONSE_GETWAVEFORMREPORTERROR']._serialized_start=14686
  _globals['_GETWAVEFORMREPORTRESPONSE_GETWAVEFORMREPORTERROR']._serialized_end=14727
  _globals['_JOBSERVICE']._serialized_start=14748
  _globals['_JOBSERVICE']._serialized_end=17810
# @@protoc_insertion_point(module_scope)



from qm._loc import _get_loc
from qm.grpc.qm.pb import inc_qua_pb2
from qm.qua._scope_management.scopes_manager import scopes_manager
from qm.qua._expressions import QuaVariable, QuaGlobalVarXor, QuaGlobalVarRead


def global_var_xor(*bits: int) -> QuaGlobalVarXor:
    return QuaGlobalVarXor(bits)


def global_var_read(*bits: int, shift: bool = False) -> QuaGlobalVarRead:
    return QuaGlobalVarRead(bits, shift)


def assign_global_var(*variables: QuaVariable[bool]) -> None:
    statement = inc_qua_pb2.QuaProgram.AnyStatement(
        globalVariableAssignment=inc_qua_pb2.QuaProgram.GlobalVariableAssignmentStatement(
            loc=_get_loc(), variables=[v.unwrapped_scalar for v in variables]
        )
    )
    scopes_manager.append_statement(statement)



from typing import List, Tuple, cast

from qm.grpc.qm.pb import frontend_pb2
from qm.api.models.capabilities import ServerCapabilities
from qm.simulate.interface import SimulatorInterface, SupportedConnectionTypes


class RawInterface(
    SimulatorInterface[frontend_pb2.ExecutionRequest.Simulate.SimulationInterface.RawInterface.Connections]
):
    """Creates a raw interface for use in [qm.simulate.interface.SimulationConfig][].
    A raw interface defines samples that will be inputted into the OPX inputs.

    Args:
        connections (list):

            List of tuples with the connection. Each tuple should be:

                ``(toController: str, toFEM: int, toPort: int, toSamples: List[float])``
        noisePower: How much noise to add to the input.

    Example:
        ```python
        job = qmm.simulate(config, prog, SimulationConfig(
                          duration=20000,
                          # 500 ns of DC 0.2 V into con1 input 1
                          simulation_interface=RawInterface([("con1", 1, [0.2]*500)])
        ```
    """

    @classmethod
    def _validate_and_standardize_single_connection(
        cls, connection: SupportedConnectionTypes, fem_number_in_simulator: int
    ) -> frontend_pb2.ExecutionRequest.Simulate.SimulationInterface.RawInterface.Connections:
        if not isinstance(connection, tuple):
            raise Exception("each connection must be of type tuple")
        if len(connection) == 4:
            cls._validate_connection_type(
                connection, [str, int, str, list], "(from_controller, from_fem, from_port, to_samples)"
            )
            tuple_4 = cast(Tuple[str, int, int, List[float]], connection)
            return frontend_pb2.ExecutionRequest.Simulate.SimulationInterface.RawInterface.Connections(
                fromController=tuple_4[0], fromFem=tuple_4[1], fromPort=tuple_4[2], toSamples=tuple_4[3]
            )
        if len(connection) == 3:
            cls._validate_connection_type(connection, [str, int, list], "(from_controller, from_port, to_samples)")
            tuple_3 = cast(Tuple[str, int, List[float]], connection)
            return frontend_pb2.ExecutionRequest.Simulate.SimulationInterface.RawInterface.Connections(
                fromController=tuple_3[0],
                fromFem=fem_number_in_simulator,
                fromPort=tuple_3[1],
                toSamples=tuple_3[2],
            )
        raise Exception("connection should be tuple of length of 3 or 4")

    def update_simulate_request(
        self, request: frontend_pb2.SimulationRequest, capabilities: ServerCapabilities
    ) -> frontend_pb2.SimulationRequest:
        request.simulate.simulationInterface.raw.CopyFrom(
            frontend_pb2.ExecutionRequest.Simulate.SimulationInterface.RawInterface(
                noisePower=self.noisePower,
                connections=self._validate_and_standardize_connections(
                    self._raw_connections, capabilities.fem_number_in_simulator
                ),
            )
        )
        return request



import warnings
import dataclasses
from typing import Set, Optional, FrozenSet, Collection

from qm.api.models.info import QuaMachineInfo
from qm.exceptions import UnsupportedCapabilitiesError, CapabilitiesNotInitializedError


@dataclasses.dataclass(frozen=True)
class Capability:
    qop_name: str
    from_qop_version: Optional[str] = None
    name_in_exception: Optional[str] = None

    @property
    def unsupported_exception_message(self) -> str:
        if self.name_in_exception:
            capability_name = self.name_in_exception
        else:
            capability_name = self.qop_name.replace("qm.", "")
            capability_name = capability_name.replace("_", " ")

        if self.from_qop_version is None:
            return f"{capability_name} is not supported in the installed QOP version."
        else:
            return f"{capability_name} is supported from QOP {self.from_qop_version} and above."


class QopCaps:
    job_streaming_state = Capability("qm.job_streaming_state")
    multiple_inputs_for_element = Capability("qm.multiple_inputs_for_element")
    analog_delay = Capability("qm.analog_delay")
    shared_oscillators = Capability("qm.shared_oscillators")
    crosstalk = Capability("qm.crosstalk")
    shared_ports = Capability("qm.shared_ports")
    input_stream = Capability("qm.input_stream")
    new_grpc_structure = Capability("qm.new_grpc_structure")
    double_frequency = Capability("qm.double_frequency")
    command_timestamps = Capability("qm.play_tag", "2.2", "timestamping commands")
    inverted_digital_output = Capability("qm.inverted_digital_output")
    sticky_elements = Capability("qm.sticky_elements")
    octave_reset = Capability("qm.octave_reset")
    fast_frame_rotation = Capability("qm.fast_frame_rotation", "2.2")
    keeping_dc_offsets = Capability("qm.keep_dc_offsets_when_closing")
    octave_management = Capability("support_octave_mgmnt", "2.5")

    # QOP3
    qop3 = Capability("__qop3", "3.0")
    opx1000_fems_return_1_based = Capability("1_based_fem", "3.0")
    waveform_report_endpoint = Capability("qm.waveform_report_endpoint", "3.3")
    exponential_iir_filter = Capability("qm.exponential_iir_filter", "3.3")
    broadcast = Capability("qm.broadcast", "3.3")
    chunk_streaming = Capability("qm.chunk_streaming", "3.3")
    fast_frame_rotation_deprecated = Capability("qm.fast_frame_rotation_deprecated", "3.3")
    config_v2 = Capability("qm.config_v2", "3.5")
    waveform_array = Capability("qm.waveform_array", "3.5")
    exponential_dc_gain_filter = Capability("qm.exponential_dc_gain_filter", "3.5")
    multiple_streams_fetching = Capability("qm.multiple_streams_fetching", "3.5")
    external_stream = Capability("qm.external_stream", "3.5", name_in_exception="declaring an external stream")
    device_temperatures = Capability("qm.device_temperatures", "3.6")
    port_voltage_limits = Capability("qm.port_voltage_limits", "3.7")
    lo_mode = Capability("qm.lo_mode", "3.7")

    @staticmethod
    def get_all() -> Set[Capability]:
        # Some built in methods are also in the class dictionary, so the 'if isinstance' filters them out
        return set(cap for cap in QopCaps.__dict__.values() if isinstance(cap, Capability))

    @staticmethod
    def qop2_caps() -> Set[Capability]:
        return set(cap for cap in QopCaps.get_all() if not cap.from_qop_version or float(cap.from_qop_version) < 3)


OPX_FEM_IDX = 1


class ServerCapabilities:
    def __init__(self, supported_capabilities: Collection[Capability]) -> None:
        self._supported_capabilities = frozenset(supported_capabilities)

    # These properties exist because the previous implementation of ServerCapabilities (used until this refactor in Jan 2025)
    # included a separate property for each capability. While most usages have been updated to call the 'supports'
    # function, these specific properties are still widely used in multiple places. To avoid extensive changes, their
    # usage was retained.
    supports_double_frequency = property(lambda self: self.supports(QopCaps.double_frequency))
    supports_sticky_elements = property(lambda self: self.supports(QopCaps.sticky_elements))

    @property
    def supported_capabilities(self) -> FrozenSet[Capability]:
        return self._supported_capabilities

    # This property is defined as a function to explicitly specify a return type, ensuring compatibility with mypy.
    @property
    def fem_number_in_simulator(self) -> int:
        return OPX_FEM_IDX if self.supports(QopCaps.qop3) else 0

    def supports(self, capability: Capability) -> bool:
        return capability in self._supported_capabilities

    def validate(self, capabilities: Collection[Capability]) -> None:
        """
        Validates if the capabilities passed are supported by the server.
        Raises an UnsupportedCapabilityError for the capabilities which are not supported.
        """
        if self.supports(QopCaps.fast_frame_rotation_deprecated) and QopCaps.fast_frame_rotation in capabilities:
            warnings.warn(
                "The fast_frame_rotation is deprecated as it is no longer faster than frame_rotation_2pi "
                "(and in fact, it is less efficient). It will be removed in future versions.",
                DeprecationWarning,
            )

        unsupported_capabilities = set(capabilities) - self._supported_capabilities
        if unsupported_capabilities:
            exception_message = "\nAlso: ".join(
                [capability.unsupported_exception_message for capability in unsupported_capabilities]
            )
            raise UnsupportedCapabilitiesError(exception_message)

    @classmethod
    def build(cls, qua_implementation: Optional[QuaMachineInfo] = None) -> "ServerCapabilities":
        qop_caps = qua_implementation.capabilities if qua_implementation is not None else list()
        supported_capabilities = set(cap for cap in QopCaps.get_all() if cap.qop_name in qop_caps)

        if QopCaps.qop3.qop_name in qop_caps:
            # Doing this in case all the QOP2 caps are not added by default when it is a QOP3 machine
            supported_capabilities.update(QopCaps.qop2_caps())

        return cls(supported_capabilities)


class _OfflineCapabilities(ServerCapabilities):
    """
    This class allows us to use some of the conversions to protobuf and QUA script generation also when user is not connected to the QOP.
    Before this class existed, the data was stored in a singleton that was kept after the user disconnected from the QOP.
    In the future, there is a chance we won't need this class, as the models will be independent of the capabilities.
    """

    def __init__(self) -> None:
        super().__init__(frozenset())
        self._is_set = False

    def supports(self, capability: Capability) -> bool:
        if not self._is_set:
            raise CapabilitiesNotInitializedError
        return super().supports(capability)

    def set(self, capabilities: Collection[Capability]) -> None:
        self._supported_capabilities = frozenset(capabilities)
        self._is_set = True

    def set_from_implementation(self, implementation: Optional[QuaMachineInfo] = None) -> None:
        """Since we want to keep a single instance of this class, we don't want to use the build function that returns a new instance"""
        tmp = ServerCapabilities.build(implementation)
        self.set(tmp.supported_capabilities)


offline_capabilities = (
    _OfflineCapabilities()
)  # "offline" is a bit misleading as this is used in some functions as the current capabilities.



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: qm/pb/errors.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'qm/pb/errors.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12qm/pb/errors.proto\x12\x0eqm.grpc.errors*\xa7\x02\n\x14JobManagerErrorTypes\x12\x1e\n\x1aJobManagerUnspecifiedError\x10\x00\x12\x14\n\x0fMissingJobError\x10\xe8\x07\x12#\n\x1eInvalidJobExecutionStatusError\x10\xe9\x07\x12(\n#InvalidOperationOnSimulatorJobError\x10\xea\x07\x12#\n\x1eInvalidOperationOnRealJobError\x10\xeb\x07\x12\x1e\n\x19JobOperationSpecificError\x10\xec\x07\x12\x15\n\x10\x43onfigQueryError\x10\xed\x07\x12\x1c\n\x17UnknownInputStreamError\x10\xee\x07\"\x05\x08\x01\x10\xe7\x07\"\t\x08\xd0\x0f\x10\xff\xff\xff\xff\x07*\xba\x02\n\x1eJobOperationSpecificErrorTypes\x12 \n\x1cJobOperationUnspecifiedError\x10\x00\x12\x1c\n\x17SingleInputElementError\x10\xb8\x17\x12!\n\x1cInvalidCorrectionMatrixError\x10\xb9\x17\x12-\n(ElementWithoutIntermediateFrequencyError\x10\xba\x17\x12&\n!InvalidDigitalInputThresholdError\x10\xbb\x17\x12%\n InvalidDigitalInputDeadtimeError\x10\xbc\x17\x12%\n InvalidDigitalInputPolarityError\x10\xbd\x17\"\x05\x08\x01\x10\xb7\x17\"\t\x08\xa0\x1f\x10\xff\xff\xff\xff\x07*\xa0\x01\n\x15\x43onfigQueryErrorTypes\x12\x1f\n\x1b\x43onfigQueryUnspecifiedError\x10\x00\x12\x1b\n\x16MissingControllerError\x10\xa0\x1f\x12\x18\n\x13MissingElementError\x10\xa1\x1f\x12\x1d\n\x18MissingDigitalInputError\x10\xa2\x1f\"\x05\x08\x01\x10\x9f\x1f\"\t\x08\x88\'\x10\xff\xff\xff\xff\x07\x42%\n\x0eqm.grpc.errorsB\x0b\x45rrorsProtoP\x01\xa2\x02\x03HLWb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qm.pb.errors_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\016qm.grpc.errorsB\013ErrorsProtoP\001\242\002\003HLW'
  _globals['_JOBMANAGERERRORTYPES']._serialized_start=39
  _globals['_JOBMANAGERERRORTYPES']._serialized_end=334
  _globals['_JOBOPERATIONSPECIFICERRORTYPES']._serialized_start=337
  _globals['_JOBOPERATIONSPECIFICERRORTYPES']._serialized_end=651
  _globals['_CONFIGQUERYERRORTYPES']._serialized_start=654
  _globals['_CONFIGQUERYERRORTYPES']._serialized_end=814
# @@protoc_insertion_point(module_scope)



from qm.grpc.octave.v1 import api_pb2
from qm.octave_sdk._octave_client import OctaveClient


# This function is shared between the octave and the octave_conflicts module, so it moved to this shared location
def _get_synth_state(client: OctaveClient, synth_index: int) -> api_pb2.SynthUpdate:
    response = client.acquire_module(
        api_pb2.ModuleReference(type=api_pb2.OctaveModule.OCTAVE_MODULE_SYNTHESIZER, index=synth_index)
    )
    synth_state = response.synth
    if isinstance(synth_state, api_pb2.SynthUpdate):
        return synth_state
    else:
        raise Exception("could not get synth state")



import warnings

from qm.utils import deprecation_message
from qm.simulate._simulator_samples import SimulatorSamples, SimulatorControllerSamples

__all__ = ["SimulatorSamples", "SimulatorControllerSamples"]


warnings.warn(
    deprecation_message(
        "qm.results.simulator_samples",
        deprecated_in="1.2.3",
        removed_in="2.0.0",
        details="please import `SimulatorSamples` and `SimulatorControllerSamples` directly from qm.",
    ),
    DeprecationWarning,
    stacklevel=2,
)



from qm.grpc.qm.pb import job_manager_pb2


class DeprecatedJobManagerServiceStub(object):
    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetElementCorrection = channel.unary_unary(
            "/qm.grpc.jobManager.JobManagerService/GetElementCorrection",
            request_serializer=job_manager_pb2.GetElementCorrectionRequest.SerializeToString,
            response_deserializer=job_manager_pb2.GetElementCorrectionResponse.FromString,
            _registered_method=True,
        )

        self.SetElementCorrection = channel.unary_unary(
            "/qm.grpc.jobManager.JobManagerService/SetElementCorrection",
            request_serializer=job_manager_pb2.SetElementCorrectionRequest.SerializeToString,
            response_deserializer=job_manager_pb2.SetElementCorrectionResponse.FromString,
            _registered_method=True,
        )

        self.InsertInputStream = channel.unary_unary(
            "/qm.grpc.jobManager.JobManagerService/InsertInputStream",
            request_serializer=job_manager_pb2.InsertInputStreamRequest.SerializeToString,
            response_deserializer=job_manager_pb2.InsertInputStreamResponse.FromString,
            _registered_method=True,
        )



import logging
import contextlib
from typing import (
    List,
    Type,
    Tuple,
    Union,
    Literal,
    Mapping,
    TypeVar,
    Iterable,
    Optional,
    Protocol,
    Sequence,
    Generator,
    TypedDict,
    MutableSequence,
    cast,
    overload,
)

from qm.type_hinting import Number
from qm.utils import LOG_LEVEL_MAP
from qm.octave import QmOctaveConfig
from qm.api.v2.base_api_v2 import BaseApiV2
from qm.program import Program, load_config
from qm.elements_db import init_octave_elements
from qm.octave.octave_manager import OctaveManager
from qm.simulate.interface import SimulationConfig
from qm.grpc.qm.grpc.v2 import qm_api_pb2, qmm_api_pb2
from qm.api.models.server_details import ConnectionDetails
from qm.utils.config_utils import get_controller_pb_config
from qm.utils.protobuf_utils import proto_repeated_to_list
from qm.api.simulation_api import create_simulation_request
from qm.grpc.qm.grpc.v2.qm_api_pb2_grpc import QmServiceStub
from qm.api.v2.job_api.simulated_job_api import SimulatedJobApi
from qm.elements.up_converted_input import UpconvertedInputNewApi
from qm.api.models.capabilities import QopCaps, ServerCapabilities
from qm.program._dict_to_pb_converter import DictToQuaConfigConverter
from qm.program._fill_defaults_in_config_v1 import fill_defaults_in_config_v1
from qm.octave.qm_octave import QmOctaveForNewApi, create_dc_offset_octave_update
from qm.grpc.qm.pb import compiler_pb2, frontend_pb2, qm_manager_pb2, inc_qua_config_pb2
from qm.api.v2.job_api.job_api import JobApi, JobData, JobStatus, transfer_statuses_to_enum
from qm.type_hinting.config_types import FullQuaConfig, MixerConfigType, LogicalQuaConfig, ControllerQuaConfig
from qm.api.models.compiler import CompilerOptionArguments, standardize_compiler_params, get_request_compiler_options
from qm.octave.octave_mixer_calibration import (
    AutoCalibrationParams,
    MixerCalibrationResults,
    LOFrequencyCalibrationResult,
    NewApiOctaveMixerCalibration,
)
from qm.exceptions import (
    QopResponseError,
    CompilationException,
    JobNotFoundException,
    QopConfigValidationError,
    OctaveUnsupportedOnUpdate,
    FailedToExecuteJobException,
    UnsupportedCapabilitiesError,
)

logger = logging.getLogger(__name__)


def _log_messages(messages: MutableSequence[compiler_pb2.CompilerMessage]) -> None:
    for message in messages:
        logger.log(LOG_LEVEL_MAP[message.level], message.message)


class ErrorResponseWithConfigValidationErrors(Protocol):
    config_validation_errors: List[qm_manager_pb2.ConfigValidationMessage]
    messages: List[compiler_pb2.CompilerMessage]


def get_formatted_errors(error: ErrorResponseWithConfigValidationErrors) -> str:
    # Check if the exception is due to config validation errors
    if error.config_validation_errors:
        formatted_errors = str(QopConfigValidationError(error.config_validation_errors))
        logger.error(formatted_errors)
        return formatted_errors
    else:
        # Check if the exception is due to some other reason
        error_messages = []
        for msg in error.messages:
            lvl = LOG_LEVEL_MAP[msg.level]
            if lvl == logging.ERROR:
                error_messages.append(msg.message)
            logger.log(lvl, msg.message)

        return "\n".join(error_messages)


E = TypeVar("E", bound=Exception)


@contextlib.contextmanager
def handle_qop_error(custom_error: E) -> Generator[None, None, None]:
    """
    A context manager that catches QopResponseError exceptions and enriches the `custom_error` with additional details
    about the cause.
    """
    try:
        yield
    except QopResponseError as qop_error:
        logger.error(str(custom_error))

        try:
            formatted_errors = get_formatted_errors(qop_error.error)
            raise type(custom_error)(
                f"{str(custom_error)}. See the following errors:\n{formatted_errors}"
            ) from qop_error
        # In case the error doesn't have the expected fields (defined in the  ErrorResponseWithConfigValidationErrors
        # protocol)
        except AttributeError:
            raise custom_error


@contextlib.contextmanager
def handle_simulation_error() -> Generator[None, None, None]:
    with handle_qop_error(FailedToExecuteJobException("Job failed. Failed to execute program")):
        yield


class QmApi(BaseApiV2[QmServiceStub]):
    SIMULATED_JOB_CLASS = SimulatedJobApi

    def __init__(
        self,
        connection_details: ConnectionDetails,
        qm_id: str,
        capabilities: ServerCapabilities,
        octave_config: Optional[QmOctaveConfig],
        octave_manager: OctaveManager,
        pb_config: Optional[inc_qua_config_pb2.QuaConfig] = None,
    ) -> None:
        # todo - remove _pb_config when octave config is in the GW
        super().__init__(connection_details)
        self._caps = capabilities
        self._id = qm_id
        pb_config = pb_config or self._get_pb_config()
        self._elements = init_octave_elements(pb_config, capabilities, octave_config)
        self._octave_manager = octave_manager
        self._octave = QmOctaveForNewApi(self, octave_manager)
        self._octave_already_configured = get_controller_pb_config(pb_config).octaves != {}

    @property
    def octave(self) -> QmOctaveForNewApi:
        return self._octave

    @property
    def id(self) -> str:
        """Hopefully temporary, till we move to the new API."""
        return self._id

    @property
    def _stub_class(self) -> Type[QmServiceStub]:
        return QmServiceStub

    def _convert_config_param_to_pb(
        self, config: Optional[Union[FullQuaConfig, LogicalQuaConfig]], function_name: str
    ) -> Optional[inc_qua_config_pb2.QuaConfig]:
        """
        Validates the config_v2 capability and converts the config to proto format.
        """
        if config:
            self._validate_capability_for_config_param(function_name)
            return self._load_config_with_exception_handling(config, function_name)
        return None

    def _validate_capability_for_config_param(self, function_name: str) -> None:
        """
        Validates that there is a config_v2 capability, since only from config_v2 (QOP 3.5) it is possible to pass a
        config to the api functions (which use this function for validation).
        """
        if not self._caps.supports(QopCaps.config_v2):
            raise UnsupportedCapabilitiesError(
                f"Passing a config to qm.{function_name}() is supported from QOP {QopCaps.config_v2.from_qop_version} and above."
            )

    def _load_config_with_exception_handling(
        self, config: Union[FullQuaConfig, ControllerQuaConfig, LogicalQuaConfig], function_name: str
    ) -> inc_qua_config_pb2.QuaConfig:
        try:
            return load_config(config, init_mode=False, octave_already_configured=self._octave_already_configured)
        except OctaveUnsupportedOnUpdate as e:
            raise OctaveUnsupportedOnUpdate(
                "Changing the octave parameters in an open QM is currently not "
                f"supported for qm.{function_name}. Please use the qm.octave functions instead."
            ) from e

    def update_config(self, config: ControllerQuaConfig) -> None:
        """Updates the controller config in the `QuantumMachine`.
        The controller config includes the "controllers", "octaves" & "mixers" sections.
        Updating the config will only update future jobs that were not executed or compiled yet.
        It will also update the idle values (e.g. DC Offset) which will be active between programs.

        Args:
            config: The physical config
        """
        config_pb = self._load_config_with_exception_handling(config, self.update_config.__name__)
        request = qm_api_pb2.UpdateConfigRequest(quantum_machine_id=self._id, config=config_pb)
        self._run(self._stub.UpdateConfig, request, timeout=self._timeout)

    def get_jobs(
        self,
        job_ids: Iterable[str] = tuple(),
        user_ids: Iterable[str] = tuple(),
        description: str = "",
        status: Union[JobStatus, Iterable[JobStatus]] = tuple(),
    ) -> List[JobData]:
        """
        Get jobs based on filtering criteria. All fields are optional.

        Args:
            job_ids: A list of jobs ids
            user_ids: A list of user ids
            description: Jobs' description
            status: A list of job statuses

        Returns:
            A list of jobs
        """
        query_params = qmm_api_pb2.JobsQueryParams(
            quantum_machine_ids=[self._id],
            job_ids=list(job_ids),
            user_ids=list(user_ids),
            description=description,
            status=transfer_statuses_to_enum(status),
        )
        request = qm_api_pb2.QmServiceGetJobsRequest(query=query_params)
        response: qmm_api_pb2.GetJobsSuccess = self._run(self._stub.GetJobs, request, timeout=self._timeout)
        return [JobData.from_grpc(j) for j in response.jobs]

    def _get_pb_config(self) -> inc_qua_config_pb2.QuaConfig:
        request = qm_api_pb2.QmServiceGetConfigRequest(quantum_machine_id=self._id)
        response: qm_api_pb2.GetConfigSuccess = self._run(self._stub.GetConfig, request, timeout=self._timeout)
        config = response.config
        fill_defaults_in_config_v1(config)
        return config

    def get_config(self) -> FullQuaConfig:
        """Gets the current config of the qm

        Returns:
            A dictionary with the QMs config
        """
        converter = DictToQuaConfigConverter(self._caps)
        return converter.deconvert(self._get_pb_config())

    def compile(
        self,
        program: Program,
        compiler_options: Optional[CompilerOptionArguments] = None,
        *,
        config: Optional[Union[FullQuaConfig, LogicalQuaConfig]] = None,
    ) -> str:
        """Compiles a QUA program to be executed later. The returned `program_id`
        can then be directly added to the queue. For a detailed explanation
        see [Precompile Jobs](../Guides/features.md#precompile-jobs).

        Args:
            program: A QUA program
            compiler_options: Optional arguments for compilation
            config: -- Available from QOP 3.5 --
                The configuration used with the program. The logical config is required if it was not supplied to the `QuantumMachine`.
                A full configuration (containing both logical and controller configs), can be used to override the default `QuantumMachine` settings.

        Returns:
            a program_id str

        Example:
            ```python
            program_id = qm.compile(program)
            job = qm.add_to_queue(program_id)
            job.wait_until("running")
            ```
        """
        self._caps.validate(program.used_capabilities)

        pb_config = self._convert_config_param_to_pb(config, self.compile.__name__)

        if compiler_options is None:
            compiler_options = CompilerOptionArguments()
        program.qua_program.compilerOptions.CopyFrom(get_request_compiler_options(compiler_options))
        request = qm_api_pb2.QmServiceCompileRequest(
            quantum_machine_id=self._id, high_level_program=program.qua_program, config=pb_config
        )
        with handle_qop_error(CompilationException("Compilation failed")):
            response: qm_api_pb2.CompileResponse.CompilationSuccess = self._run(
                self._stub.Compile, request, timeout=self._timeout
            )

        _log_messages(response.messages)
        return response.program_id

    def _add_compiled(self, program_id: str) -> JobApi:
        request = qm_api_pb2.QmServiceAddCompiledToQueueRequest(quantum_machine_id=self._id, program_id=program_id)
        response: qm_api_pb2.AddCompiledToQueueResponse.AddCompiledToQueueResponseSuccess = self._run(
            self._stub.AddCompiledToQueue, request, timeout=self._timeout
        )
        return self._get_job(response.job_id)

    def _add_program(self, program: Program, config: Optional[inc_qua_config_pb2.QuaConfig]) -> JobApi:
        request = qm_api_pb2.QmServiceAddToQueueRequest(
            quantum_machine_id=self._id, high_level_program=program.qua_program, config=config
        )
        with handle_qop_error(FailedToExecuteJobException("Failed to execute program")):
            response: qm_api_pb2.AddToQueueResponse.AddToQueueResponseSuccess = self._run(
                self._stub.AddToQueue, request, timeout=self._timeout
            )

        _log_messages(response.messages)
        return self._get_job(response.job_id)

    @overload
    def add_to_queue(self, program: str) -> JobApi:
        pass

    @overload
    def add_to_queue(
        self,
        program: Program,
        *,
        config: Optional[Union[FullQuaConfig, LogicalQuaConfig]] = None,
        compiler_options: Optional[CompilerOptionArguments] = None,
    ) -> JobApi:
        pass

    def add_to_queue(
        self,
        program: Union[Program, str],
        *,
        config: Optional[Union[FullQuaConfig, LogicalQuaConfig]] = None,
        compiler_options: Optional[CompilerOptionArguments] = None,
    ) -> JobApi:
        """
        Adds a QmJob to the queue.
        Programs in the queue will play as soon as possible.

        Args:
            program: A QUA program or a compiled program id
            config: -- Available from QOP 3.5 --
                The configuration used with the program. The logical config is required if it was not supplied to the `QuantumMachine`.
                A full configuration (containing both logical and controller configs), can be used to override the default `QuantumMachine` settings.
            compiler_options: Optional arguments for compilation

        Returns:
            A job object
        """
        logger.info("Adding program to queue.")

        if isinstance(program, str):
            if compiler_options:
                raise ValueError("Cannot add compiler options to a compiled program.")
            if config is not None:
                raise ValueError("Cannot add a config to a compiled program.")
            job = self._add_compiled(program)
        else:
            self._caps.validate(program.used_capabilities)
            pb_config = self._convert_config_param_to_pb(config, self.add_to_queue.__name__)

            if compiler_options is None:
                compiler_options = CompilerOptionArguments()
            program.qua_program.compilerOptions.CopyFrom(get_request_compiler_options(compiler_options))
            job = self._add_program(program, pb_config)

        logger.info(f"Program added to queue. Job id: {job.id}")
        return job

    def get_job(self, job_id: str) -> JobApi:
        """
        Get a job based on the job_id.

        Args:
            job_id: A list of jobs ids

        Returns:
            The job
        """
        jobs_data = self.get_jobs(job_ids=[job_id])
        if not jobs_data:
            raise JobNotFoundException(job_id)
        else:
            job_data = jobs_data[0]
            if self._caps.supports(QopCaps.waveform_report_endpoint) and job_data.is_simulation:
                return self._get_simulated_job(job_id)

            return self._get_job(job_id)

    def _get_job(self, job_id: str) -> JobApi:
        return JobApi(self.connection_details, job_id, self._caps)

    def _get_simulated_job(
        self, job_id: str, simulated: Union[frontend_pb2.SimulatedResponsePart, None] = None
    ) -> SimulatedJobApi:
        return self.SIMULATED_JOB_CLASS(
            self.connection_details, job_id, simulated_response=simulated, capabilities=self._caps
        )

    def _create_simulate_request(
        self,
        program: Program,
        simulate: SimulationConfig,
        compiler_options: Optional[CompilerOptionArguments] = None,
        *,
        config: Optional[Union[FullQuaConfig, LogicalQuaConfig]] = None,
        strict: Optional[bool] = None,
        flags: Optional[List[str]] = None,
    ) -> qm_api_pb2.QmServiceSimulateRequest:
        """
        Creates a simulate request.
        """
        standardized_compiler_options = standardize_compiler_params(compiler_options, strict, flags)
        pb_config = self._convert_config_param_to_pb(config, self.simulate.__name__)

        # We put self._get_pb_config() as the value for the config argument of create_simulation_request(), just
        # because we had to fill in something so it would work. We are not going to use that value, since in the new
        # request the config could be None. This is why when building the request we use the "pb_config" variable
        # instead of the "standard_request.config".
        standard_request = create_simulation_request(
            self._get_pb_config(), program, simulate, standardized_compiler_options, self._caps
        )
        return qm_api_pb2.QmServiceSimulateRequest(
            quantum_machine_id=self._id,
            high_level_program=standard_request.highLevelProgram,
            simulate=standard_request.simulate,
            controller_connections=standard_request.controllerConnections,
            config=pb_config,
        )

    def simulate(
        self,
        program: Program,
        simulate: SimulationConfig,
        compiler_options: Optional[CompilerOptionArguments] = None,
        *,
        config: Optional[Union[FullQuaConfig, LogicalQuaConfig]] = None,
        strict: Optional[bool] = None,
        flags: Optional[List[str]] = None,
    ) -> SimulatedJobApi:
        """Simulates the outputs of a deterministic QUA program.

        Equivalent to ``execute()`` with ``simulate=SimulationConfig`` (see example).

        Note:
            A simulated job does not support calling QuantumMachine API functions.

        Args:
            program: A QUA ``program()`` object to execute
            simulate: If given, will be simulated instead of executed.
            compiler_options: Optional arguments for compilation.
            config: -- Available from QOP 3.5 --
                The configuration used with the program. The logical config is required if it was not supplied to the `QuantumMachine`.
                A full configuration (containing both logical and controller configs), can be used to override the default `QuantumMachine` settings.
            strict: This parameter is deprecated, please use `compiler_options`
            flags: This parameter is deprecated, please use `compiler_options`

        Returns:
            A ``QmJob`` object (see Job API).
        """
        self._caps.validate(program.used_capabilities)

        logger.info("Simulating program.")
        request = self._create_simulate_request(
            program, simulate, compiler_options, config=config, strict=strict, flags=flags
        )
        with handle_simulation_error():
            response: qmm_api_pb2.SimulationSuccess = self._run(self._stub.Simulate, request, timeout=self._timeout)

        _log_messages(response.messages)

        return self._get_simulated_job(response.job_id, response.simulated)

    def execute(
        self,
        program: Program,
        config: Optional[Union[FullQuaConfig, LogicalQuaConfig]] = None,
        *,
        compiler_options: Optional[CompilerOptionArguments] = None,
    ) -> JobApi:
        """Closes all running jobs in the QM, clears the queue, executes a program, wait for it to start,
        and returns a job object.

        Note:

            Calling execute will halt any currently running program and clear the current
            queue. If you want to add a job to the queue, use qm.queue.add()

        Args:
            program: A QUA ``program()`` object to execute
            config: -- Available from QOP 3.5 --
                The configuration used with the program. The logical config is required if it was not supplied to the `QuantumMachine`.
                A full configuration (containing both logical and controller configs), can be used to override the default `QuantumMachine` settings.
            compiler_options: Optional arguments for compilation.

        Returns:
            A ``QmJob`` object (see Job API).
        """
        # todo: change to an API call that stops programs, clears queue, executes, and returns job once it's running
        raise NotImplementedError()

    def clear_queue(
        self,
        job_ids: Iterable[str] = tuple(),
        user_ids: Iterable[str] = tuple(),
        description: str = "",
        status: Union[JobStatus, Iterable[JobStatus]] = tuple(),
    ) -> List[str]:
        """
        Clears jobs from the queue based on filtering criteria. All fields are optional.

        Args:
            job_ids: A list of jobs ids
            user_ids: A list of user ids
            description: Jobs' description
            status: A list of job statuses

        Returns:
            A list of the removed jobs ids
        """
        query_params = qmm_api_pb2.JobsQueryParams(
            quantum_machine_ids=[self._id],
            job_ids=list(job_ids),
            user_ids=list(user_ids),
            description=description,
            status=transfer_statuses_to_enum(status),
        )
        response: qm_api_pb2.RemoveJobsResponse.RemoveJobsResponseSuccess = self._run(
            self._stub.RemoveJobs, qm_api_pb2.RemoveJobsRequest(query=query_params), timeout=self._timeout
        )
        return proto_repeated_to_list(response.removed_job_ids)

    def close(self) -> None:
        """
        Closes the quantum machine.
        """
        self._run(
            self._stub.Close, qm_api_pb2.QmServiceCloseRequest(quantum_machine_id=self._id), timeout=self._timeout
        )

    def get_queue_count(self) -> int:
        """
        Get the number of jobs currently on the queue

        Returns:
            The number of jobs in the queue
        """
        jobs = self.get_jobs(status=["In queue"])
        return len(jobs)

    def calibrate_element(
        self,
        qe: str,
        lo_if_dict: Optional[Mapping[float, Sequence[float]]] = None,
        save_to_db: bool = True,
        params: Optional[AutoCalibrationParams] = None,
    ) -> MixerCalibrationResults:
        """Calibrate the up converters associated with a given element for the given LO & IF frequencies.

        - Frequencies can be given as a dictionary with LO frequency as the key and a list of IF frequencies for every LO
        - If no frequencies are given calibration will occur according to LO & IF declared in the element
        - The function need to be run for each element separately
        - The results are saved to a database for later use

        Args:
            qe (str): The name of the element for calibration
            lo_if_dict ([Mapping[float, Tuple[float, ...]]]): a dictionary with LO frequency as the key and
                a list of IF frequencies for every LO
            save_to_db (bool): If true (default), The calibration
                parameters will be saved to the calibration database
            params: Optional calibration parameters
        """

        inst = self._elements[qe]

        if params is None:
            params = AutoCalibrationParams()

        inst_input = inst.input
        assert isinstance(inst_input, UpconvertedInputNewApi)

        if lo_if_dict is None:
            lo_if_dict = {inst_input.lo_frequency: (inst.intermediate_frequency,)}
        client = self._octave_manager._get_client_from_port(inst_input.port)
        res = NewApiOctaveMixerCalibration(client=client, qm_api=self).calibrate(
            element=inst,
            lo_if_dict=lo_if_dict,
            params=params,
        )

        if save_to_db:
            calibration_db = self._octave_manager._octave_config._calibration_db
            if calibration_db is None:
                logger.warning("No calibration db found, can't save results")
            else:
                calibration_db.update_calibration_result(res, inst_input.port, "auto")

        key = (inst_input.lo_frequency, cast(float, inst_input.gain))
        if key in res:
            qe_cal = res[key]
            update: ControllerQuaConfig = {}
            if inst.intermediate_frequency in qe_cal.image:
                update = create_dc_offset_octave_update(inst_input, i_offset=qe_cal.i0, q_offset=qe_cal.q0)

            update["mixers"] = {inst_input.mixer: self._add_mixer_corrections(qe_cal, inst_input)}
            self.update_config(update)

        return res

    def _add_mixer_corrections(
        self, qe_cal: LOFrequencyCalibrationResult, inst_input: UpconvertedInputNewApi
    ) -> List[MixerConfigType]:
        """
        Before QOP 3.5, there was no need to add the mixers that were used when opening the QM.
        Starting with 3.5, however, the introduction of “send program with config” changed this behavior: the list
        of mixers is now completely overridden by the configuration provided in either update_config or in the
        config sent with the program. So now we need to make sure that the mixer correction for the original IF
        frequency of the element is also added, in case it was updated as part of the calibration.
        """
        correction_entries = self._get_current_mixer_entries(inst_input.mixer)

        for if_freq, if_cal in qe_cal.image.items():
            self._upsert_correction_entry(correction_entries, if_freq, inst_input.lo_frequency, if_cal.fine.correction)

        return correction_entries

    def _get_current_mixer_entries(self, mixer_name: str) -> List[MixerConfigType]:
        current_config = self.get_config()
        mixers = current_config.get("mixers", {})
        return list(mixers.get(mixer_name, []))

    def _upsert_correction_entry(
        self,
        current_entries: List[MixerConfigType],
        intermediate_frequency: Number,
        lo_frequency: Number,
        values: Tuple[float, float, float, float],
    ) -> None:

        new_entry: MixerConfigType = {
            "correction": values,
            "intermediate_frequency": intermediate_frequency,
            "lo_frequency": lo_frequency,
        }

        # Find an existing entry with the same intermediate_frequency and lo_frequency
        idx = next(
            (
                i
                for i, e in enumerate(current_entries)
                if e.get("intermediate_frequency") == intermediate_frequency and e.get("lo_frequency") == lo_frequency
            ),
            None,
        )

        if idx is None:
            # Not present → append
            current_entries.append(new_entry)
        else:
            # Present → replace
            current_entries[idx] = new_entry

    def reset_digital_filters(self) -> None:
        """
        Reset the digital filters' state, specifically to remove the “memory” of the high-pass filter.
        See [High-Pass Compensation Filter](../Guides/output_filter.md#high-pass-compensation-filter)
        for more information.
        """
        if not self._caps.supports(QopCaps.exponential_dc_gain_filter):
            raise UnsupportedCapabilitiesError(
                f"qm.reset_digital_filters() is supported from QOP {QopCaps.exponential_dc_gain_filter.from_qop_version} and above."
            )

        self._run(
            self._stub.ResetDigitalFilters,
            qm_api_pb2.ResetDigitalFiltersRequest(quantum_machine_id=self._id),
            timeout=self._timeout,
        )


class NoRunningQmJob(Exception):
    pass


IoValue = TypedDict(
    "IoValue",
    {"io_number": Literal[1, 2], "int_value": int, "fixed_value": float, "boolean_value": bool},
    total=False,
)



from abc import ABC
from typing import Type, Optional, cast, overload

import numpy as np

from qm.exceptions import QmQuaException
from qm.type_hinting.general import Number
from qm.qua._expressions import QuaVariable
from qm.qua._dsl.variable_handling import declare
from qm.qua._dsl.scope_functions import for_, for_each_
from qm.qua.extensions.qua_iterators.qua_iterators_base import IterableBase
from qm.qua.extensions.qua_iterators.qua_iterators_types import (
    T,
    MetaDataType,
    QuaIteratorType,
    QuaIterableArrayType,
    QuaIterableArrayInput,
    QuaIterableArrayInputInt,
    QuaIterableArrayInputFloat,
)

FIXED_TYPE_TOLERANCE = 1e-10


class QuaIterableBase(IterableBase[T], ABC):
    def __init__(
        self,
        name: str,
        dtype: Type[T],
        metadata: Optional[MetaDataType],
    ):
        super().__init__(name, metadata)
        self._dtype: Type[T] = dtype

    @property
    def dtype(self) -> Type[T]:
        """
        Type of the QUA variable declared for this iterable.
        """
        return self._dtype

    def declare_var(self) -> QuaVariable[T]:
        """
        Declare the QUA variable used by this iterable.
        """
        return declare(self._dtype)

    @property
    def is_qua_iterable(self) -> bool:
        return True


class QuaIterableRange(QuaIterableBase[T]):
    """
    QUA-side range iterator.

    Use this helper when a sweep axis should execute on the QOP as a
    [`for_`][qm.qua.for_] loop rather than in Python.

    ``QuaIterableRange(name, stop)`` behaves like ``range(stop)``.
    ``QuaIterableRange(name, start, stop, step)`` behaves like
    ``range(start, stop, step)`` for integers and like
    ``numpy.arange(start, stop, step)`` for floats. ``stop`` is exclusive.

    The yielded value is a QUA variable, not a Python scalar.

    For floating-point ranges, the internal stop value may be adjusted slightly
    to avoid fixed-point boundary issues in the generated loop.

    Example:
        ```python
        with program() as prog:
            for amp_scale in QuaIterableRange("amp_scale", 0.1, 1.0, 0.05):
                play("pulse" * amp(amp_scale), "element")
        ```

        This is equivalent to:

        ```python
        with program() as prog:
            amp_scale = declare(fixed)
            with for_(amp_scale, 0.1, amp_scale < 1.0, amp_scale + 0.05):
                play("pulse" * amp(amp_scale), "element")
        ```
    """

    @overload
    def __init__(self: "QuaIterableRange[int]", name: str, stop: int, *, metadata: Optional[MetaDataType] = None):
        """
        support range(5)
        """
        ...

    @overload
    def __init__(
        self: "QuaIterableRange[int]",
        name: str,
        start: int,
        stop: int,
        step: int = 1,
        *,
        metadata: Optional[MetaDataType] = None,
    ):
        ...

    @overload
    def __init__(
        self: "QuaIterableRange[float]",
        name: str,
        start: Number,
        stop: Number,
        step: Number = 1,
        *,
        metadata: Optional[MetaDataType] = None,
    ):
        ...

    def __init__(self, name: str, /, *args: Number, metadata: Optional[MetaDataType] = None):  # type: ignore[misc]
        self._start: Number
        self._stop: Number
        self._step: Number = 1

        if len(args) == 1:
            self._stop = args[0]
            self._start = 0
        elif len(args) == 2:
            self._start, self._stop = args
        elif len(args) == 3:
            self._start, self._stop, self._step = args
        else:
            raise QmQuaException(f"not supported args len = {len(args)}")

        if isinstance(self._start, float) or isinstance(self._stop, float) or isinstance(self._step, float):
            self._start = float(self._start)
            self._stop = float(self._stop)
            self._step = float(self._step)
            # because of fixed point accuracy issues change stop in case its start + step * n == stop
            if self._start + (len(self.values)) * self._step >= self._stop - FIXED_TYPE_TOLERANCE:
                self._stop = self._stop - self._step / 2

            # Mypy doesn't propagate the self-type from overloads into the implementation
            # body, so it can't verify float/int matches T.
            # The overloads guarantee T matches the arg types at the call site.
            super().__init__(name, float, metadata)  # type: ignore[arg-type]
        else:
            self._start = int(self._start)
            self._stop = int(self._stop)
            self._step = int(self._step)
            super().__init__(name, int, metadata)  # type: ignore[arg-type]

    @property
    def values(self) -> QuaIterableArrayType[T]:
        """
        Return the Python values represented by this iterable.
        """
        return np.arange(self._start, self._stop, self._step).tolist()  # type: ignore[return-value]

    def __iter__(self) -> QuaIteratorType[T]:
        var = self.declare_var()

        with for_(var, self._start, var < self._stop, var + self._step):  # type: ignore[operator]
            self._add_to_current_scope()
            yield var
            self._set_averaged_streams()


class QuaIterable(QuaIterableBase[T]):
    """
    QUA-side iterable over an explicit sequence of numeric values.

    Use this helper when a sweep axis should execute on the QOP rather than in
    Python.

    If the values are uniformly spaced, the iterator may be optimized to a
    [`for_`][qm.qua.for_] loop. Otherwise it is compiled as a
    [`for_each_`][qm.qua.for_each_] loop.

    Integer sequences produce integer QUA variables. Any non-integer values are
    converted to float.

    The yielded value is a QUA variable, not a Python scalar.

    Example:
        ```python
        with program() as prog:
            for amp_scale in QuaIterable("amp_scale", np.linspace(0.1, 0.6, 10)):
                play("pulse" * amp(amp_scale), "element")
        ```

        This is equivalent to:

        ```python
        with program() as prog:
            amp_scale = declare(fixed)
            with for_each_(amp_scale, np.linspace(0.1, 0.6, 10)):
                play("pulse" * amp(amp_scale), "element")
        ```
    """

    @overload
    def __init__(
        self: "QuaIterable[int]",
        name: str,
        array: QuaIterableArrayInputInt,
        *,
        metadata: Optional[MetaDataType] = None,
    ):
        ...

    @overload
    def __init__(
        self: "QuaIterable[float]",
        name: str,
        array: QuaIterableArrayInputFloat,
        *,
        metadata: Optional[MetaDataType] = None,
    ):
        ...

    def __init__(
        self,
        name: str,
        array: QuaIterableArrayInput,
        *,
        metadata: Optional[MetaDataType] = None,
    ):
        raw_array = array.tolist() if isinstance(array, np.ndarray) else list(array)
        dtype: type = int if all([type(v) == int for v in raw_array]) else float
        if dtype is float:
            self._array: QuaIterableArrayType[T] = cast(QuaIterableArrayType[T], [float(v) for v in raw_array])
            # Mypy doesn't propagate the self-type from overloads into the implementation
            # body, so it can't verify float/int matches T.
            # The overloads guarantee T matches the arg types at the call site.
            super().__init__(name, float, metadata)  # type: ignore[arg-type]
        else:
            self._array = cast(QuaIterableArrayType[T], [int(v) for v in raw_array])
            super().__init__(name, int, metadata)  # type: ignore[arg-type]

        self._qua_range_itr: Optional[QuaIterableRange[T]] = None
        # according to dtype set the minimum step that can be done
        minimum_step = FIXED_TYPE_TOLERANCE if dtype is float else 1
        first_step = array[1] - array[0]
        # in case all steps in the array are up < minimum steps than we can use for.
        # the reason to use this approach is that if array is created by linspace and his likes the steps won't be ==
        optimize_to_for = all(
            [abs((array[i + 1] - array[i]) - first_step) < minimum_step for i in range(len(array) - 1)]
        )

        if optimize_to_for:
            range_itr: QuaIterableRange[T] = QuaIterableRange(  # type: ignore[assignment]
                name, array[0], array[-1] + first_step, first_step, metadata=metadata
            )
            # checking that optimization succeeded
            if range_itr.buffer_size == self.buffer_size:
                self._qua_range_itr = range_itr

    @property
    def values(self) -> QuaIterableArrayType[T]:
        """
        Return the Python values represented by this iterable.
        """
        return self._array

    def __iter__(self) -> QuaIteratorType[T]:
        # in case exist equivalent qua range itr iterate over it for optimization purposes
        if self._qua_range_itr is not None:
            yield from self._qua_range_itr
        else:
            var = self.declare_var()

            with for_each_(var, self._array):
                self._add_to_current_scope()
                yield var
                self._set_averaged_streams()

    def is_stream_averaged(self, stream_name: str) -> bool:
        """
        Check if stream is averaged on this iterable
        """
        if self._qua_range_itr is not None:
            return self._qua_range_itr.is_stream_averaged(stream_name)
        else:
            return super().is_stream_averaged(stream_name)
